#pragma ident "$Id: main.c,v 1.5 2012/02/28 17:35:00 dechavez Exp $"
/*======================================================================
 *
 * Generate condensed APS output file, compute average azimuth
 *
 *====================================================================*/
#include "util.h"

#define NFIX 3
#define NSV  13

extern char *VersionIdentString;

typedef struct {
    int nsv;
    int fix;
    char *tstamp;
    char *lat;
    char *dlat;
    char *lon;
    char *dlon;
    REAL32 azimuth;
} READING;

static READING reading;

static void help(char *myname)
{
    fprintf(stderr,"usage: %s < input > output", myname);
    exit(1);
}

static void discard(char *input)
{
    fprintf(stderr, "IGNORE: %s\n", input);
}

static void ProcessPLTIT(char **token, int ntoken, char *input)
{
int expected = 7;
static char *fid = "ProcessPLTIT";

    if (ntoken != expected) {
        fprintf(stderr, "%s: expected %d tokens, got %d\n", fid, expected, ntoken);
        discard(input);
        return;
    }

    reading.azimuth = atof(token[2]);
}

static BOOL ProcessGPGGA(char **token, int ntoken, char *input)
{
int expected = 15;
static char *fid = "ProcessGPGGA";

    if (ntoken != expected && strcasecmp(token[10], "M") != 0) {
        fprintf(stderr, "%s: expected %d tokens, got %d\n", fid, expected, ntoken);
        discard(input);
        return FALSE;
    }

    reading.tstamp = token[1];
    reading.lat = token[2];
    reading.dlat = token[3];
    reading.lon = token[4];
    reading.dlon = token[5];
    reading.fix = atoi(token[6]);
    reading.nsv = atoi(token[7]);

    if (reading.fix >= NFIX || reading.nsv >= NSV) {
        fprintf(stderr, "%s: toss fix=%d, nsv=%d\n", fid, reading.fix, reading.nsv);
        discard(input);
        return FALSE;
    }

    return TRUE;
}

static void ClearReading()
{
    memset(&reading, 0, sizeof(READING));
}

static void SaveReading(LNKLST *list)
{
#ifdef PRINT_READINGS
    printf("%s ", reading.tstamp);
    printf("%s%s ", reading.lat, reading.dlat);
    printf("%s%s ", reading.lon, reading.dlon);
    printf("%d ", reading.fix);
    printf("%2d ", reading.nsv);
    printf("%5.1f\n", reading.azimuth);
#endif /* PRINT_READINGS */

    if (!listAppend(list, &reading, sizeof(READING))) {
        perror("listAppend");
        exit(1);
    }
}

static BOOL Match(int want, int have)
{
    if (have == -1) return TRUE;
    return (have == want) ? TRUE : FALSE;
}

static void ComputeMean(LNKLST *list, int fix, int nsv)
{
LNKLST_NODE *crnt;
READING *entry;
int nsamp;
REAL64 sum, dev, sumsq, mean, stdev;

    nsamp = 0;
    sum = dev = sumsq = mean = stdev = 0.0;

    crnt = listFirstNode(list);
    while (crnt != NULL) {
        entry = (READING *) crnt->payload;
        if (Match(entry->fix, fix) && Match(entry->nsv, nsv)) {
            sum += entry->azimuth;
            ++nsamp;
        }
        crnt = listNextNode(crnt);
    }
    if (nsamp == 0) return;

    mean = sum / (REAL64) nsamp;

    crnt = listFirstNode(list);
    while (crnt != NULL) {
        entry = (READING *) crnt->payload;
        if (Match(entry->fix, fix) && Match(entry->nsv, nsv)) {
            dev = entry->azimuth - mean;
            sumsq += (dev * dev);
        }
        crnt = listNextNode(crnt);
    }
    stdev = sqrt(sumsq / (REAL64) (nsamp - 1));

    if (fix == -1 && nsv == -1) {
        printf("for all readings         ");
    } else {
        printf("for fix=%d, satellites=%d, ", fix, nsv);
    }
    printf("azimuth = %5.1llf +/- %.1llf ",  mean, stdev);
    printf("(%d readings)\n", nsamp);
}

int main (int argc, char **argv)
{
#define MAXLINELEN 256
#define COMMENT    '#'
#define DELIMITERS ","
#define MAX_TOKEN   32
int i, nchar;
char input[MAXLINELEN+1];
char copy[MAXLINELEN+1];
char *token[MAX_TOKEN];
static char *PLTIT = "$PLTIT";
static char *GPGGA = "$GPGGA";
int status, ntoken;
LNKLST *list;
int fix, nsv;

    nchar = printf("%s version %s\n", argv[0], VersionIdentString);
    for (i = 0; i < nchar; i++) printf("-"); printf("\n");

    if ((list = listCreate()) == NULL) {
        perror("listCreate");
        exit(1);
    }

    ClearReading();
    while ((status = utilGetLine(stdin, input, MAXLINELEN, COMMENT, NULL)) == 0) {
        strcpy(copy, input);
        ntoken = utilParse(copy, token, DELIMITERS, MAX_TOKEN, 0);
        if (strcmp(token[0], PLTIT) == 0) {
            ProcessPLTIT(token, ntoken, input);
        } else if (strcmp(token[0], GPGGA) == 0) {
            if (ProcessGPGGA(token, ntoken, input)) SaveReading(list);
        } else {
            discard(input);
        }
    }

    if (status != 1) {
        perror("utilGetLine");
        exit(1);
    }

    for (fix = 0; fix < NFIX; fix++) for (nsv = 0; nsv < NSV; nsv++) ComputeMean(list, fix, nsv);
    ComputeMean(list, -1, -1);

    exit(0);
}

/* Revision History
 *
 * $Log: main.c,v $
 * Revision 1.5  2012/02/28 17:35:00  dechavez
 * print out version ident
 *
 * Revision 1.4  2012/02/27 22:53:59  dechavez
 * Fixed bug where final average was just the final reading(!).  Allow for GPGGA
 * strings with less than 15 tokens as long as the units field is "M".
 *
 * Revision 1.3  2011/12/15 21:36:43  dechavez
 * don't print out reformatted readings, compute ave/stdev for all valid readings
 *
 * Revision 1.2  2011/12/15 18:32:32  dechavez
 * Fix bug computing mean, and prevent divide by zero error
 *
 * Revision 1.1  2011/11/29 22:22:53  dechavez
 * first release
 *
 */
