#pragma ident "$Id: idaspan.c,v 1.9 2011/07/07 22:30:13 dechavez Exp $"
/*======================================================================
 *
 *  Print time range spanned by IDA data, gzip data automatically
 *  detected and decompresssed.
 *
 *====================================================================*/
#include "util.h"
#include "ida.h"
#include "ida10.h"
#include "zlib.h"

typedef struct {
    int yr;
    int da;
    int hr;
    int mn;
    int sc;
    int ms;
    BOOL host;
    double value;
} TSTAMP;

void help(char *myname)
{
    fprintf(stderr,"\n");
    fprintf(stderr,"usage: %s rev=code < data_stream (gzip OK)\n", myname);
    fprintf(stderr,"\n");
    exit(1);
}

static TSTAMP SetTstamp(double dtag, IDA_EXTRA *extra)
{
TSTAMP tstamp;
int yr, da, hr, mn, sc, ms;

    tstamp.value = dtag;

    utilTsplit(tstamp.value, &tstamp.yr, &tstamp.da, &tstamp.hr, &tstamp.mn, &tstamp.sc, &tstamp.ms);

    if (extra->valid && tstamp.yr < 2000) {
        tstamp.value = (double) (extra->tstamp + SAN_EPOCH_TO_1970_EPOCH);
        utilTsplit(tstamp.value, &tstamp.yr, &tstamp.da, &tstamp.hr, &tstamp.mn, &tstamp.sc, &tstamp.ms);
        tstamp.host = TRUE;
    } else {
        tstamp.host = FALSE;
    }

    return tstamp;
}

static void SetString(char *string, TSTAMP *tstamp)
{
    if (!tstamp->host) {
        sprintf(string, "%04d:%03d-%02d:%02d", tstamp->yr, tstamp->da, tstamp->hr, tstamp->mn);
    } else {
        sprintf(string, "%04d:%03d-HH:HM", tstamp->yr, tstamp->da);
    }
}

BOOL DecodeMKx(char *buffer, int rev, TSTAMP *tofs, TSTAMP *tols, IDA *ida)
{
IDA_DHDR hdr;

    if (ida_rtype(buffer, rev) != IDA_DATA) return FALSE;
    if (ida_dhead(ida, &hdr, buffer) != 0) return FALSE;

    *tofs = SetTstamp(hdr.beg.tru, &hdr.extra);
    *tols = SetTstamp(hdr.end.tru, &hdr.extra);

    return TRUE;
}

BOOL DecodeIDA10(char *buffer, TSTAMP *tofs, TSTAMP *tols)
{
IDA10_TSHDR hdr;

    if (ida10Type((UINT8 *) buffer) != IDA10_TYPE_TS) return FALSE;
    if (ida10UnpackTSHdr((UINT8 *) buffer, &hdr) == 0) return FALSE;

    *tofs = SetTstamp(hdr.tofs, &hdr.cmn.extra);
    *tols = SetTstamp(hdr.tols, &hdr.cmn.extra);

    return TRUE;
}

int main(int argc, char **argv)
{
char *dbspec=NULL, *log=NULL, *path=NULL;
int i, debug=-1, rev=-1, type;
BOOL first=TRUE;
TSTAMP tofs, tols;
struct {
    TSTAMP tstamp;
    char string[64];
} oldest, yngest;
IDA ida;
gzFile gz;
FILE *fp = stdin;
UINT8 buffer[IDA10_MAXRECLEN];
static char *defdir = "/usr/nrts";

    idaInitHandle(&ida);

    if (argc < 2) {
        fprintf(stderr, "%s: bad arg count\n", argv[0]);
        help(argv[0]);
    }

    for (i = 1; i < argc; i++) {
        if (strncmp(argv[i], "rev=", strlen("rev=")) == 0) {
            rev = atoi(argv[i]+strlen("rev="));
        } else if (strncmp(argv[i], "debug=", strlen("debug=")) == 0) {
            debug = atoi(argv[i] + strlen("debug="));
        } else if (strncmp(argv[i], "if=", strlen("if=")) == 0) {
            path = argv[i] + strlen("if=");
        } else if (strncmp(argv[i], "log=", strlen("log=")) == 0) {
            log = argv[i] + strlen("log=");
        } else if (strncmp(argv[i], "db=", strlen("db=")) == 0) {
            dbspec = argv[i] + strlen("db=");
        } else {
            fprintf(stderr, "%s: unrecognized argument '%s'\n", argv[0], argv[i]);
            help(argv[0]);
        }
    }

    if (path != NULL && (fp = fopen(path, "rb")) == NULL) {
        fprintf(stderr, "%s: fopen: %s: %s\n", argv[0], path, strerror(errno));
        exit(1);
    }
    if ((gz = gzdopen(fileno(fp), "rb")) == NULL) {
        fprintf(stderr, "%s: gzdopen: %s\n", argv[0], strerror(errno));
        exit(1);
    }

    if (rev < 0) {
        fprintf(stderr, "%s: must specify data format rev code!\n", argv[0]);
        help(argv[0]);
    }
    ida.rev.value = rev;

    if (debug > 0) util_logopen(log, 1, 9, debug, NULL, argv[0]);

/*  Select decoder function/handle */

    if (rev < 10) {
        if ((ida.db = dbioOpen(dbspec, NULL)) == NULL) {
            fprintf(stderr, "%s: dbioOpen: %s\n", argv[0], strerror(errno));
            exit(1);
        }
    } else {
        ida.db = NULL;
    }

/* Read/decode until done */

    strcpy(oldest.string, "<undetermined>");
    strcpy(yngest.string, "<undetermined>");

    if (rev < 10) {
        while (gzread(gz, buffer, IDA_BUFLEN) == IDA_BUFLEN) {
            if (DecodeMKx(buffer, rev, &tofs, &tols, &ida)) {
                if (first) {
                    oldest.tstamp = tofs;
                    yngest.tstamp = tols;
                    first = FALSE;
                } else {
                    if (tofs.value < oldest.tstamp.value) oldest.tstamp = tofs;
                    yngest.tstamp = tols;
                }
            }
        }
    } else {
        while (ida10ReadGz(gz, buffer, IDA10_MAXRECLEN, &type, FALSE) > 0) {
            if (DecodeIDA10(buffer, &tofs, &tols)) {
                if (first) {
                    oldest.tstamp = tofs;
                    yngest.tstamp = tols;
                    first = FALSE;
                } else {
                    if (tofs.value < oldest.tstamp.value) oldest.tstamp = tofs;
                    yngest.tstamp = tols;
                }
            }
        }
    }
    
    if (!first) {
        SetString(oldest.string, &oldest.tstamp);
        SetString(yngest.string, &yngest.tstamp);
    }

    printf("%s %s\n", oldest.string, yngest.string);

    exit(0);
}

/* Revision History
 *
 * $Log: idaspan.c,v $
 * Revision 1.9  2011/07/07 22:30:13  dechavez
 * added usage error messages
 *
 * Revision 1.8  2011/03/17 20:29:29  dechavez
 * use host tstamp to determine start time when data time stamps are clearly bad
 *
 * Revision 1.7  2008/03/21 02:39:43  dechavez
 * fixed uninitialized pointer problem (always there but not visible until ported to FreeBSD)
 *
 * Revision 1.6  2006/02/10 01:49:05  dechavez
 * dbio support
 *
 * Revision 1.5  2005/10/20 19:57:29  dechavez
 * added support for gzip'd files, updated default dbspec
 *
 * Revision 1.4  2004/06/21 20:14:09  dechavez
 * changed time_tag "true" field to "tru"
 *
 * Revision 1.3  2003/12/10 06:31:22  dechavez
 * cosmetic changes to calm solaris cc
 *
 * Revision 1.2  2002/03/11 19:17:35  dec
 * changed string for unknown time to <undetermined>
 *
 * Revision 1.1  2002/02/15 21:50:31  dec
 * created
 *
 */
