#pragma ident "$Id: tokens.c,v 1.2 2012/06/24 18:28:56 dechavez Exp $"
/*======================================================================
 *
 *  Set or restore data port configuration
 *
 *====================================================================*/
#include "qcal.h"

static UINT32 delay = 250;
static int ntry = 10;

BOOL ConfigureDataPort(QCAL *qcal)
{
QDP *qdp;
int errcode;
UINT16 type;
Q330_LCQ *lcq;
static char *fid = "ConfigureDataPortTokens";

    if (qcal->dport.tokens == NULL) {
        LogMsg("%s configuration left unchanged\n", qdpPortString(qcal->port));
        return TRUE;
    }

    switch (qcal->port) {
      case QDP_LOGICAL_PORT_1: type = QDP_MEM_TYPE_CONFIG_DP1; break;
      case QDP_LOGICAL_PORT_2: type = QDP_MEM_TYPE_CONFIG_DP2; break;
      case QDP_LOGICAL_PORT_3: type = QDP_MEM_TYPE_CONFIG_DP3; break;
      case QDP_LOGICAL_PORT_4: type = QDP_MEM_TYPE_CONFIG_DP4; break;
      default:
        fprintf(stderr, "*** UNEXPECTED ERROR *** port %d is not a valid data port!\n", qcal->port);
        return FALSE;
    }

    if ((lcq = q330LookupLcq(qcal->dport.tokens, qcal->cfg)) == NULL) {
        fprintf(stderr, "ERROR: no such tokenset '%s'\n", qcal->dport.tokens);
        return FALSE;
    }

/* Register with the configuration port */

    if ((qdp = q330Register(&qcal->addr, QDP_CFG_PORT, qcal->debug, NULL, &errcode)) == NULL) {
        if (errcode == QDP_ERR_BUSY) {
            fprintf(stderr, "%s %s already in use", qcal->addr.connect.ident, qdpPortString(QDP_CFG_PORT));
            fprintf(stderr, "You wouldn't have a Willard session running, would you?");
        } else {
            fprintf(stderr, "Unable to register with configuration port: %s", qdpErrcodeString(errcode));
        }
        return FALSE;
    }

/* Save a copy of the exisiting configuration */

    if (!qdp_C1_RQFIX(qdp, &qcal->fix)) {
        fprintf(stderr, "ERROR: %s: qdp_C1_RQFIX: %s \n", fid, strerror(errno));
        return DeregisterAndReturn(qdp, FALSE);
    }

    if (!qdpRequestDportConfig(qdp, &qcal->dport.saved, qcal->port)) {
        fprintf(stderr, "ERROR: %s: qdpRequestDportConfig: %s \n", fid, strerror(errno));
        return DeregisterAndReturn(qdp, FALSE);
    }

/* Get a working copy to modify */

    if (!qdpRequestDportConfig(qdp, &qcal->dport.work, qcal->port)) {
        fprintf(stderr, "ERROR: %s: qdpRequestDportConfig: %s \n", fid, strerror(errno));
        return DeregisterAndReturn(qdp, FALSE);
    }

/* Modify the tokens and update the configuration */

    if (!q330SetTokenLcq(&qcal->dport.work.token, lcq, qcal->dport.sta, qcal->dport.net)) {
        fprintf(stderr, "ERROR: %s: q330SetTokenLcq: %s \n", fid, strerror(errno));
        return DeregisterAndReturn(qdp, FALSE);
    }

    if (!qdpConfigureDataPort(qdp, &qcal->dport.work, qcal->port, &qcal->fix, delay, ntry)) {
        fprintf(stderr, "ERROR: %s: qdpConfigureDataPort: %s \n", fid, strerror(errno));
        return DeregisterAndReturn(qdp, FALSE);
    }

/* Success */

    LogMsg("%s configured with %s tokens\n", qdpPortString(qcal->port), qcal->dport.tokens);
    return DeregisterAndReturn(qdp, TRUE);
}

BOOL RestoreDataPort(QCAL *qcal)
{
QDP *qdp;
int errcode;
static char *fid = "RestoreDataPort";

    if (qcal->dport.tokens == NULL) return TRUE; /* never changed */

/* Register with the configuration port */

    if ((qdp = q330Register(&qcal->addr, QDP_CFG_PORT, qcal->debug, NULL, &errcode)) == NULL) {
        LogMsg("Unable to register with configuration port: %s", qdpErrcodeString(errcode));
        LogMsg("%s tokens have NOT been restored to their original state!\n", qdpPortString(qcal->port));
        return FALSE;
    }

    if (!qdpConfigureDataPort(qdp, &qcal->dport.saved, qcal->port, &qcal->fix, delay, ntry)) {
        fprintf(stderr, "ERROR: %s: qdpConfigureDataPort: %s \n", fid, strerror(errno));
        return DeregisterAndReturn(qdp, FALSE);
    }

    LogMsg("%s restored\n", qdpPortString(qcal->port));
    return DeregisterAndReturn(qdp, TRUE);
}

/*-----------------------------------------------------------------------+
 |                                                                       |
 | Copyright (C) 2011 Regents of the University of California            |
 |                                                                       |
 | This software is provided 'as-is', without any express or implied     |
 | warranty.  In no event will the authors be held liable for any        |
 | damages arising from the use of this software.                        |
 |                                                                       |
 | Permission is granted to anyone to use this software for any purpose, |
 | including commercial applications, and to alter it and redistribute   |
 | it freely, subject to the following restrictions:                     |
 |                                                                       |
 | 1. The origin of this software must not be misrepresented; you must   |
 |    not claim that you wrote the original software. If you use this    |
 |    software in a product, an acknowledgment in the product            |
 |    documentation of the contribution by Project IDA, UCSD would be    |
 |    appreciated but is not required.                                   |
 | 2. Altered source versions must be plainly marked as such, and must   |
 |    not be misrepresented as being the original software.              |
 | 3. This notice may not be removed or altered from any source          |
 |    distribution.                                                      |
 |                                                                       |
 +-----------------------------------------------------------------------*/

/* Revision History
 *
 * $Log: tokens.c,v $
 * Revision 1.2  2012/06/24 18:28:56  dechavez
 * Modified to use libqdp 3.4.0 where data port and tokens are bound together
 * and main channel output frequencies are forced to agree with token set
 *
 * Revision 1.1  2011/04/14 19:22:35  dechavez
 * created
 *
 */
