#pragma ident "$Id: ida10.c,v 1.10 2011/11/07 17:42:59 dechavez Exp $"
/*======================================================================
 *
 *  Callback routine for writing IDA10 packets to disk loop
 *
 *====================================================================*/
#include "qhlp.h"

#define MY_MOD_ID QHLP_MOD_IDA10

void SaveIDA10(DISK_LOOP_DATA *output, QDP_HLP *hlp)
{
static BOOL first = TRUE;
static char *fid = "SaveIDA10";

    logioUpdateWatchdog(Watch, fid);

    if (!qdpHlpToIDA10(output->raw.payload, hlp, output->dl->sys->seqno.counter)) {
        LogMsg("%s: qdpHlpToIDA10 failed: %s\n", fid, strerror(errno));
        Exit(MY_MOD_ID + 1);
    }

    output->options.offset.seqno = ida10SeqnoOffset(output->raw.payload);
    if (!isidlWriteToDiskLoop(output->dl, &output->raw, &output->options)) {
        LogMsg("%s: isidlWriteToDiskLoop failed: %s\n", fid, strerror(errno));
        Exit(MY_MOD_ID + 2);
    }

    if (first) {
        LogMsg("initial IDA10 packet completed");
        first = FALSE;
    }
}

BOOL InitIDA10(QDP_HLP_RULES *rules, DISK_LOOP_DATA *output, UINT32 flags)
{
    output->raw.hdr.desc.comp = ISI_COMP_NONE;
    output->raw.hdr.desc.type = ISI_TYPE_IDA10;
    output->raw.hdr.desc.order = ISI_TYPE_UNDEF;
    output->raw.hdr.desc.size = sizeof(UINT8);
    output->raw.hdr.status = ISI_RAW_STATUS_OK;
    output->options.flags = ISI_OPTION_GENERATE_SEQNO | ISI_OPTION_INSERT_32BIT_SEQNO;

    return qdpInitHLPRules(
        rules,
        IDA10_DEFDATALEN,
        QDP_HLP_FORMAT_NOCOMP32,
        ProcessHLP,
        (void *) output,
        flags
    );
}

void ParoIDA10(DISK_LOOP_DATA *output, UINT8 *payload)
{
#define MAXSAMP 240
static BOOL first = TRUE;
int i, nsamp;
UINT8 *ptr, descriptor;
QDPLUS_PAROSCI *paro;
QDPLUS_USERPKT userpkt;
IDA10_GENTAG gentag;
INT16 fact, mult;
INT32 prev, value;
REAL64 dsint;
char chnloc[IDA10_CNAMLEN+1];
static char *fid = "ParoIDA10";
IDA10_TS ts;

    qdplusDecodeUser(payload, &userpkt);
    if (userpkt.type != QDPLUS_DT_USER_PAROSCI) {
        LogMsg("%s: unsupported DT_USER type %d", fid, userpkt.type);
        return;
    }
    paro = &userpkt.data.parosci; /* just to make the code easier to read */

    if ((nsamp = paro->ndiff + 1) > MAXSAMP) {
        LogMsg("%s: too many samples to process (%d > %d)", fid, nsamp, MAXSAMP);
        return;
    }

/* Convert from QDPLUS_GENTAG to IDA10_GENTAG (could just cast but this is "safer") */

    gentag.tstamp = paro->tofs.tstamp;
    gentag.status.receiver = paro->tofs.status.receiver;
    gentag.status.generic = paro->tofs.status.generic;

/* Build the IDA10 header */

    ptr = output->raw.payload;
    memset(ptr, 0, output->raw.hdr.len.payload);
    ptr += utilPackBytes(ptr, (UINT8 *) "TS", 2);
    *ptr++ = 10; /* format 10 */
    *ptr++ = 8;  /* subformat 5 */
    ptr += utilPackBytes(ptr, output->site, IDA105_SNAME_LEN);
    ptr += utilPackBytes(ptr, output->netid, IDA105_NNAME_LEN);
    ptr += ida10PackGenericTtag(ptr, &gentag);
    ptr += utilPackUINT32(ptr, 0); /* will be seqno */
    ptr += utilPackUINT32(ptr, time(NULL) - SAN_EPOCH_TO_1970_EPOCH);
    ptr += IDA108_RESERVED_BYTES;
    ptr += utilPackUINT16(ptr, (IDA10_TSHEADLEN - IDA10_CMNHEADLEN) + (nsamp * sizeof(INT32)));

    /* end of common header */

    /* stream name, a concatenation of channel and location codes */

    sprintf(chnloc, "%s%s", paro->chn, paro->loc);
    utilTrimString(chnloc);
    util_lcase(chnloc);
    ptr += utilPackBytes(ptr, chnloc, IDA10_CNAMLEN);

    /* data format/status descriptor */

    descriptor = 0;
    descriptor |= IDA10_COMP_NONE;
    descriptor |= IDA10_A2D_24;
    descriptor |= IDA10_MASK_INT32;
    *ptr++ = descriptor;

    /* conversion gain */

    *ptr++ = 1;

    /* number of samples */

    ptr += utilPackUINT16(ptr, nsamp);

    /* sample rate */

    dsint = (REAL64) (paro->sint) / (REAL64) NANOSEC_PER_SEC;
    ida10SintToFactMult(dsint, &fact, &mult);
    ptr += utilPackINT16(ptr, fact);
    ptr += utilPackINT16(ptr, mult);

    /* the data */

    ptr += utilPackINT32(ptr, paro->first);
    for (prev = paro->first, i = 0; i < paro->ndiff; i++) {
        value = prev + paro->diff[i];
        ptr += utilPackINT32(ptr, value);
        prev = value;
    }

    /* write it out */

    if (!isidlWriteToDiskLoop(output->dl, &output->raw, &output->options)) {
        LogMsg("%s: isidlWriteToDiskLoop failed: %s\n", fid, strerror(errno));
        Exit(MY_MOD_ID + 2);
    }

    if (first) {
        LogMsg("initial barometer packet completed");
        first = FALSE;
    }
}
/*-----------------------------------------------------------------------+
 |                                                                       |
 | Copyright (C) 2007 Regents of the University of California            |
 |                                                                       |
 | This software is provided 'as-is', without any express or implied     |
 | warranty.  In no event will the authors be held liable for any        |
 | damages arising from the use of this software.                        |
 |                                                                       |
 | Permission is granted to anyone to use this software for any purpose, |
 | including commercial applications, and to alter it and redistribute   |
 | it freely, subject to the following restrictions:                     |
 |                                                                       |
 | 1. The origin of this software must not be misrepresented; you must   |
 |    not claim that you wrote the original software. If you use this    |
 |    software in a product, an acknowledgment in the product            |
 |    documentation of the contribution by Project IDA, UCSD would be    |
 |    appreciated but is not required.                                   |
 | 2. Altered source versions must be plainly marked as such, and must   |
 |    not be misrepresented as being the original software.              |
 | 3. This notice may not be removed or altered from any source          |
 |    distribution.                                                      |
 |                                                                       |
 +-----------------------------------------------------------------------*/

/* Revision History
 *
 * $Log: ida10.c,v $
 * Revision 1.10  2011/11/07 17:42:59  dechavez
 * removed call to isiToMseed() (that is now handled inside isidlWriteToDiskLoop())
 *
 * Revision 1.9  2011/11/03 17:53:23  dechavez
 * Generate equivalent MiniSEED equivalent, when possible
 *
 * Revision 1.8  2011/10/12 17:44:38  dechavez
 * InitIDA10() - add ISI_OPTION_INSERT_32BIT_SEQNO to disk loop options
 * SaveIDA10() - use ida10SeqnoOffset() to locate where to insert seqno
 * ParoIDA10() - build IDA10.8 instead of IDA10.5 packets
 *
 * Revision 1.7  2011/04/07 22:55:04  dechavez
 * added ParoIDA10()
 *
 * Revision 1.6  2009/01/26 21:18:55  dechavez
 * switched to liblogio 2.4.1 syntax for watchdog loggin
 *
 * Revision 1.5  2009/01/05 17:42:28  dechavez
 * added logioUpdateWatchdog() tracers
 *
 * Revision 1.4  2007/09/22 02:33:12  dechavez
 * removed processIDA10 and changed the QDP HLP rules to use ProcessHLP instead
 *
 * Revision 1.3  2007/09/17 23:27:51  dechavez
 * checkpoint - reworking threshold trigger (now with specific list of triggered channels)
 *
 * Revision 1.2  2007/09/14 19:54:23  dechavez
 * added event detector support
 *
 * Revision 1.1  2007/05/03 21:12:28  dechavez
 * initial release
 *
 */
