#pragma ident "$Id: main.c,v 1.3 2012/05/02 18:26:04 dechavez Exp $"
/*======================================================================
 *
 *  Tristar TS-45 command and control, via Moxa gateway.
 *
 *====================================================================*/
#include <modbus.h>
#include "platform.h"
#include "tristar.h"
#include "util.h" /* for the USEC, MSEC macros */

#define DEFAULT_MOXA_IP "192.168.1.20"

typedef struct {
    char *cmd;
    int code;
} TRISTAR_CMD_MAP;

static TRISTAR_CMD_MAP cmd_map[] = {
    {         "soh", TRISTAR_ADDR_NONE          },
    { "clearAlarms", TRISTAR_COIL_CLEAR_ALARMS   },
    { "clearFaults", TRISTAR_COIL_CLEAR_FAULTS   },
    {    "clearAhR", TRISTAR_COIL_CLEAR_AH_R     },
    {    "clearAhT", TRISTAR_COIL_CLEAR_AH_T     },
    {    "clearKwh", TRISTAR_COIL_CLEAR_KWH      },
    {"clearBattSvc", TRISTAR_COIL_RESET_BATT_SVC },
    {  "disconnect", TRISTAR_COIL_DISCONNECT     },
    {      "eqtrig", TRISTAR_COIL_EQTRIG         },
    { "lvdOverride", TRISTAR_COIL_LVD_OVERRIDE   },
    {       "reset", TRISTAR_COIL_RESET          },
    {          NULL, 0                           }
};

BOOL DebugFlag = FALSE;

static void help(char *myname)
{
    fprintf(stderr, "usage: %s [moxa=dot.dec.ima.lip] [options] {1|2}=cmd\n", myname);
    fprintf(stderr, "\n");
    fprintf(stderr, "Options:\n");
    fprintf(stderr, "bto=msec     => set byte timeout\n");
    fprintf(stderr, "rto=msec     => set response timeout\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "Slave commands:\n");
    fprintf(stderr, "soh          => print state of health report\n");
    fprintf(stderr, "clearAlarms  => clear alarms\n");
    fprintf(stderr, "clearFaults  => clear faults\n");
    fprintf(stderr, "clearAhR     => clear resetable Ah\n");
    fprintf(stderr, "clearAhT     => clear total Ah\n");
    fprintf(stderr, "clearKwh     => clear KWh\n");
    fprintf(stderr, "clearBattSvc => reset battery service reminder\n");
    fprintf(stderr, "disconnect   => force control into disconnect state\n");
    fprintf(stderr, "eqtrig       => equalize triggered\n");
    fprintf(stderr, "lvdOverride  => LVD override for one cycle\n");
    fprintf(stderr, "reset        => reset control\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "default: moxa=%s\n", DEFAULT_MOXA_IP);
    exit(1);
}

static BOOL ParseCommand(char *string, int *pcode)
{
int i;
char *cmd;

    if (strlen(string) == 1) {
        *pcode = TRISTAR_ADDR_NONE;
        return TRUE;
    }
    cmd = &string[2];
    for (i = 0; cmd_map[i].cmd != NULL; i++) {
        if (strcasecmp(cmd, cmd_map[i].cmd) == 0) {
            *pcode = cmd_map[i].code;
            return TRUE;
        }
    }

    fprintf(stderr, "ERROR: unrecognized slave command string '%s'\n", string);
    return FALSE;
}

int main(int argc, char **argv)
{
modbus_t *ctx;
int i, port = MODBUS_TCP_DEFAULT_PORT, addr, nb;
char server[MAXPATHLEN+1];
struct timeval bto, rto;
UINT32 msec;
UINT16 dest[1024];
TRISTAR_RAM ram;
int command, slave = -1;

/*  Get command line arguments  */

    strncpy(server, DEFAULT_MOXA_IP, MAXPATHLEN);

    bto.tv_sec = bto.tv_usec = rto.tv_sec = rto.tv_usec = 0;

    for (i = 1; i < argc; i++) {

        if (strncmp(argv[i], "-h", strlen("-h")) == 0) {
            help(argv[0]);
        } else if (strncasecmp(argv[i], "bto=", strlen("bto=")) == 0) {
            utilMsecToTimeval(&bto, (msec = atoi(argv[i] + strlen("bto="))));
        } else if (strncasecmp(argv[i], "rto=", strlen("rto=")) == 0) {
            utilMsecToTimeval(&rto, (msec = atoi(argv[i] + strlen("rto="))));
        } else if (strncasecmp(argv[i], "moxa=", strlen("moxa=")) == 0) {
            if (!utilParseServer(argv[i]+strlen("moxa="), server, &port)) exit(1);
        } else if (strcmp(argv[1], "1") == 00 || strncasecmp(argv[i], "1=", strlen("1=")) == 0) {
            slave = 1;
            if (!ParseCommand(argv[i], &command)) exit(1);
        } else if (strcmp(argv[1], "2") == 00 || strncasecmp(argv[i], "2=", strlen("2=")) == 0) {
            slave = 2;
            if (!ParseCommand(argv[i], &command)) exit(1);
        } else if (strncmp(argv[i], "-debug", strlen("-debug")) == 0) {
            DebugFlag = TRUE;
        } else {
            fprintf(stderr, "%s: unrecognized argument '%s'\n", argv[0], argv[i]);
            help(argv[0]);
        }
    }

/* Check for required arguments */

    if (slave < 0) {
        fprintf(stderr, "ERROR: missing required slave argument\n");
        help(argv[0]);
    }

/* Connect to gateway */

    if ((ctx = modbus_new_tcp(server, port)) == NULL) {
        fprintf(stderr, "ERROR: modbus_new_tcp(%s, %d): %s\n", server, port, strerror(errno));
        exit(1);
    }
    if (DebugFlag) modbus_set_debug(ctx, TRUE);

    if (bto.tv_sec != 00 || bto.tv_usec != 0) modbus_set_byte_timeout(ctx, &bto);
    if (rto.tv_sec != 00 || rto.tv_usec != 0) modbus_set_response_timeout(ctx, &rto);

    if ((modbus_connect(ctx)) == -1) {
        fprintf(stderr, "ERROR: modbus_connect(%s): %s\n", server, modbus_strerror(errno));
        exit(1);
    }
    
/* Execute command */

    switch (command) {
      case TRISTAR_ADDR_NONE:
        tristarReadRam(ctx, slave, &ram);
        tristarPrintRam(slave, &ram);
        break;
      default:
        modbus_set_slave(ctx, slave);
        if (modbus_write_bit(ctx, command, TRUE) != 1) {
            fprintf(stderr, "ERROR: modbus_write_bit(%s, 0x%04x, TRUE): %s\n", server, command, modbus_strerror(errno));
        }
    }

    modbus_close(ctx);
}

/*-----------------------------------------------------------------------+
 |                                                                       |
 | Copyright (C) 2011 Regents of the University of California            |
 |                                                                       |
 | This software is provided 'as-is', without any express or implied     |
 | warranty.  In no event will the authors be held liable for any        |
 | damages arising from the use of this software.                        |
 |                                                                       |
 | Permission is granted to anyone to use this software for any purpose, |
 | including commercial applications, and to alter it and redistribute   |
 | it freely, subject to the following restrictions:                     |
 |                                                                       |
 | 1. The origin of this software must not be misrepresented; you must   |
 |    not claim that you wrote the original software. If you use this    |
 |    software in a product, an acknowledgment in the product            |
 |    documentation of the contribution by Project IDA, UCSD would be    |
 |    appreciated but is not required.                                   |
 | 2. Altered source versions must be plainly marked as such, and must   |
 |    not be misrepresented as being the original software.              |
 | 3. This notice may not be removed or altered from any source          |
 |    distribution.                                                      |
 |                                                                       |
 +-----------------------------------------------------------------------*/

/* Revision History
 *
 * $Log: main.c,v $
 * Revision 1.3  2012/05/02 18:26:04  dechavez
 * added bto (byte timeout) and rto (response timeout) options
 *
 * Revision 1.2  2012/04/25 21:07:53  dechavez
 * moved large chunks of code over to libtristar 1.0.0
 *
 */
