#pragma ident "$Id: fsa.h,v 1.13 2011/02/03 17:48:18 dechavez Exp $"
/*======================================================================
 * 
 * Constants for the QDP Finite State Automaton
 * 
 * Two automatons are defined here.  Command and control sessions
 * (connections on either the configuration or special functions port)
 * use the ControlFSA state machine, while data sessions (connections
 * on one of the 4 logical data ports) use DataFSA.  The reason we
 * need two is that since DT_DATA packets are not useful without
 * the metadata, we extend the registration phase to ensure that all
 * the required configuration information has been obtained before
 * turning on the data stream.   The DataFSA allows for the events
 * needed to get to this higher level state.
 *
 *====================================================================*/
#ifndef qdp_fsa_included
#define qdp_fsa_included

#ifdef __cplusplus
extern "C" {
#endif

/* states */
#define QDP_STATE_NEEDSN       0 /* need to poll for serial number */
#define QDP_STATE_WAITSN       1 /* waiting for serial number */
#define QDP_STATE_BROAD_RQSRV  2 /* registration request sent to broadcast address */
#define QDP_STATE_BROAD_SRVRSP 3 /* challenge response sent to broadcast address */
#define QDP_STATE_WAITPHY      4 /* waiting for physical interface data */
#define QDP_STATE_UNREG        5 /* unregistered */
#define QDP_STATE_RQSRV        6 /* registration request has been sent */
#define QDP_STATE_SRVRSP       7 /* challenge response has been sent */
#define QDP_STATE_REGISTERED   8 /* registered (C&C link ready) */
#define QDP_STATE_TOKEN        9 /* loading DP tokens */
#define QDP_STATE_READY       10 /* data link ready */
#define QDP_STATE_OFF         11 /* automaton is off (failed library) */
#define QDP_NUM_STATE         12
#define QDP_MAX_STATE        (QDP_NUM_STATE - 1)
#define QDP_STATE_NOCHANGE    QDP_NUM_STATE

#ifdef INCLUDE_FSA_STRINGS
static char *StateString[QDP_NUM_STATE] = {
    "Need Serial Number",
    "Waiting for Serial Number",
    "Broadcast Registration Request Sent",
    "Broadcast Challenge Response Sent",
    "Waiting for Physical Interfaces",
    "Unregistered",
    "Registration Request Sent",
    "Challenge Response Sent",
    "Registered",
    "Loading DP tokens",
    "Data Link Ready",
    "Off"
};
#endif /* INCLUDE_FSA_STRINGS */

/* events */
#define QDP_MIN_EVENT         0
#define QDP_EVENT_UP          0 /* library ready */
#define QDP_EVENT_CTO         1 /* command port timeout */
#define QDP_EVENT_DTO         2 /* data port timeout */
#define QDP_EVENT_HBEAT       3 /* time for status request */
#define QDP_EVENT_MYSN        4 /* serial number received */
#define QDP_EVENT_PHY         5 /* physcial interface data received */
#define QDP_EVENT_CONFIG      6 /* configuration data received */
#define QDP_EVENT_TOKEN_SOME  7 /* DP token packet received */
#define QDP_EVENT_TOKEN_DONE  8 /* final DP token packet received */
#define QDP_EVENT_SRVCH       9 /* server challenge received */
#define QDP_EVENT_TOOMANY    10 /* too many servers */
#define QDP_EVENT_NOTREG     11 /* not registered */
#define QDP_EVENT_CERR       12 /* general C1_ERR received */
#define QDP_EVENT_CACK       13 /* command acknowledgement received */
#define QDP_EVENT_CTRL       14 /* general control port packet received */
#define QDP_EVENT_DATA       15 /* DT_DATA packet received */
#define QDP_EVENT_FILL       16 /* DT_FILL packet received */
#define QDP_EVENT_NOSUP      17 /* unspported packet received */
#define QDP_EVENT_CMDRDY     18 /* command packet from app */
#define QDP_EVENT_CRCERR     19 /* CRC error */
#define QDP_EVENT_IOERR      20 /* I/O error */
#define QDP_EVENT_DOWN       21 /* shutdown automaton */
#define QDP_NUM_EVENT        22
#define QDP_MAX_EVENT        (QDP_NUM_EVENT - 1)

#ifdef INCLUDE_FSA_STRINGS
static char *EventString[QDP_NUM_EVENT] = {
    "Automaton Started",
    "CTO",
    "DTO",
    "Heartbeat",
    "Serial Number Received",
    "Physical Interfaces Data Received",
    "Configuration Data Received",
    "DP Token Data Received",
    "Final DP Token Data Received",
    "Server Challenge Received",
    "Too Many Servers",
    "Not Currently Registered",
    "C1_CERR Received",
    "C1_CACK Received",
    "Control port packet received",
    "DT_DATA Received",
    "DT_FILL Received",
    "Unexpected Packet Received",
    "Command Received From App",
    "CRC Error",
    "I/O Error",
    "Automaton Stopped"
};
#endif /* INCLUDE_FSA_STRINGS */

/* actions */
#define QDP_MIN_ACTION     0
#define QDP_ACTION_NONE    0 /* event processing complete */
#define QDP_ACTION_SEND    1 /* (re)send automaton command */
#define QDP_ACTION_SRVRQ   2 /* issue server request */
#define QDP_ACTION_BLDRSP  3 /* build server response */
#define QDP_ACTION_TLU     4 /* this layer up */
#define QDP_ACTION_TLD     5 /* this layer down */
#define QDP_ACTION_SLEEP   6 /* sleep */
#define QDP_ACTION_REGERR  7 /* note registration error */
#define QDP_ACTION_CERR    8 /* process C1_CERR on registered link */
#define QDP_ACTION_CACK    9 /* process C1_CACK on registered link */
#define QDP_ACTION_CTRL   10 /* process control port packet on registered link */
#define QDP_ACTION_RQCNF  11 /* request data link configuration */
#define QDP_ACTION_LDCNF  12 /* process data link configuration */
#define QDP_ACTION_RQMEM  13 /* request DP tokens */
#define QDP_ACTION_LDMEM  14 /* save DP tokens */
#define QDP_ACTION_OPEN   15 /* open data port */
#define QDP_ACTION_DTO    16 /* data port timeout processing */
#define QDP_ACTION_HBEAT  17 /* send a heartbeat */
#define QDP_ACTION_USRCMD 18 /* forward user command (if any) */
#define QDP_ACTION_DACK   19 /* acknowledge data packets, if required */
#define QDP_ACTION_DATA   20 /* process DT_DATA packet */
#define QDP_ACTION_FILL   21 /* process DT_FILL packet */
#define QDP_ACTION_DROP   22 /* drop spurious packet */
#define QDP_ACTION_FLUSH  23 /* flush reorder buffer */
#define QDP_ACTION_CRCERR 24 /* deal with CRC error */
#define QDP_ACTION_BUSY   25 /* handle busy server */
#define QDP_ACTION_FAIL   26 /* library bug, shutdown system */
#define QDP_ACTION_POLLSN 27 /* poll for serial number */
#define QDP_ACTION_LDSN   28 /* save serial number */
#define QDP_ACTION_RQPHY  29 /* request physical interfaces */
#define QDP_ACTION_LDPHY  30 /* save physical interfaces */
#define QDP_NUM_ACTION    31
#define QDP_MAX_ACTION    (QDP_NUM_ACTION - 1)

#ifdef INCLUDE_FSA_STRINGS
static char *ActionString[QDP_NUM_ACTION] = {
    "No Action (done)",
    "Send Automaton Command",
    "Issue Server Request",
    "Build Challenge Response",
    "This Layer Up",
    "This Layer Down",
    "Sleep",
    "Registration Error",
    "Process C1_CERR",
    "Process C1_CACK",
    "Process Control Port Packet",
    "Request Configuration Data",
    "Process Configuration Data",
    "Request DP Tokens",
    "Save DP Tokens",
    "Open Data Port",
    "Process Data Port Timeout",
    "Issue Status Heartbeat",
    "Process User Command",
    "Acknowledge Data Packets",
    "Process DT_DATA Packet",
    "Process DT_FILL Packet",
    "Drop Spurious Packet",
    "Flush Reorder Buffer",
    "Handle CRC error",
    "Handle busy server",
    "Abort Library",
    "Poll for Serial Number",
    "Save Serial Number",
    "Request Physical Interfaces",
    "Save Physical Interfaces"
};
#endif /* INCLUDE_FSA_STRINGS */

#define MAX_ACTION_PER_EVENT 3
typedef struct {
    struct {
        int action[MAX_ACTION_PER_EVENT]; /* actions */
        int next;                         /* new state */
    } event[QDP_NUM_EVENT];
} QDP_STATE_MACHINE;

#ifdef INCLUDE_FSA_TABLE

/* state transition table for a control (non-data) link */

static QDP_STATE_MACHINE ControlFSA[QDP_NUM_STATE] = {

    { /* ControlFSA state: need serial number */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ QDP_ACTION_POLLSN, QDP_ACTION_NONE,   -1,                QDP_STATE_WAITSN,
        /* CTO    */ -1,                -1,                -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,   
        /* Ack    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    -1,                -1,                QDP_STATE_OFF
    },

    { /* ControlFSA state: waiting for serial number */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_POLLSN, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_LDSN,   QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_BROAD_RQSRV,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,   
        /* Ack    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* ControlFSA state: broadcast server request sent */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_BLDRSP, QDP_ACTION_SEND,   QDP_ACTION_NONE,   QDP_STATE_BROAD_SRVRSP,
        /* TooMny */ QDP_ACTION_BUSY,   QDP_ACTION_NONE,   QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_REGERR, QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_REGERR, QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Ack    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },
     
    { /* ControlFSA state: broadcast server challenge response sent */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_SEND,   QDP_ACTION_NONE,   -1,                QDP_STATE_BROAD_SRVRSP,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_BLDRSP, QDP_ACTION_SEND,   QDP_ACTION_NONE,   QDP_STATE_BROAD_SRVRSP,
        /* TooMny */ QDP_ACTION_SLEEP,  QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_BROAD_RQSRV,
        /* NotReg */ QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   -1,                QDP_STATE_BROAD_RQSRV,
        /* Err    */ QDP_ACTION_REGERR, QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Ack    */ QDP_ACTION_RQPHY,  QDP_ACTION_NONE,   -1,                QDP_STATE_WAITPHY,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },
 
    { /* ControlFSA state: waiting for physical interface data */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_RQCNF,  QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Hbeat  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_LDPHY,  QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_RQSRV,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_TLD,    QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_BROAD_RQSRV,
        /* Err    */ QDP_ACTION_CERR,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ack    */ QDP_ACTION_CACK,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_CTRL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_TLD,    QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* ControlFSA state: ready */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   -1,                QDP_STATE_RQSRV,
        /* CTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,   
        /* Ack    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* ControlFSA state: server request sent */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_BLDRSP, QDP_ACTION_SEND,   QDP_ACTION_NONE,   QDP_STATE_SRVRSP,
        /* TooMny */ QDP_ACTION_BUSY,   QDP_ACTION_NONE,   QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_REGERR, QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_REGERR, QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Ack    */ QDP_ACTION_TLU,    QDP_ACTION_NONE,   -1,                QDP_STATE_REGISTERED,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* ControlFSA state: server challenge response sent */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_SEND,   QDP_ACTION_NONE,   -1,                QDP_STATE_SRVRSP,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_BLDRSP, QDP_ACTION_SEND,   QDP_ACTION_NONE,   QDP_STATE_SRVRSP,
        /* TooMny */ QDP_ACTION_SLEEP,  QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_RQSRV,
        /* NotReg */ QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   -1,                QDP_STATE_RQSRV,
        /* Err    */ QDP_ACTION_REGERR, QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Ack    */ QDP_ACTION_TLU,    QDP_ACTION_NONE,   -1,                QDP_STATE_REGISTERED,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* ControlFSA state: registered */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_USRCMD, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_CTRL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_CTRL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_CTRL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_TLD,    QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_RQSRV,
        /* Err    */ QDP_ACTION_CERR,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ack    */ QDP_ACTION_CACK,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_CTRL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_USRCMD, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_TLD,    QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* ControlFSA state: loading DP tokens (a command and control link should never get here)*/
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* PhyRcv */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CnfRcv */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Token1 */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Token2 */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Srvch  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* TooMny */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* NotReg */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Err    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Ack    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Ctrl   */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Data   */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Fill   */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Nosup  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Cmdrdy */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CRCerr */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* IOerr  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* ControlFSA state: ready (a command and control link should never get here)*/
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* PhyRcv */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CnfRcv */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Token1 */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Token2 */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Srvch  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* TooMny */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* NotReg */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Err    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Ack    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Ctrl   */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Data   */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Fill   */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Nosup  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Cmdrdy */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CRCerr */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* IOerr  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* ControlFSA state: off */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Hbeat  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* MySN   */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Ack    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    }
};

/* state transition table for a data link */

static QDP_STATE_MACHINE DataFSA[QDP_NUM_STATE] = {

    { /* DataFSA state: need serial number */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ QDP_ACTION_POLLSN, QDP_ACTION_NONE,   -1,                QDP_STATE_WAITSN,
        /* CTO    */ -1,                -1,                -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,   
        /* Ack    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* DataFSA state: waiting for serial number */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_POLLSN, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_LDSN,   QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_RQSRV,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,   
        /* Ack    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* DataFSA state: broadcast server request sent */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_BLDRSP, QDP_ACTION_SEND,   QDP_ACTION_NONE,   QDP_STATE_BROAD_SRVRSP,
        /* TooMny */ QDP_ACTION_BUSY,   QDP_ACTION_NONE,   QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_REGERR, QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_REGERR, QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Ack    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },
     
    { /* DataFSA state: broadcast server challenge response sent */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_SEND,   QDP_ACTION_NONE,   -1,                QDP_STATE_BROAD_SRVRSP,
        /* DTO    */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* Hbeat  */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_BLDRSP, QDP_ACTION_SEND,   QDP_ACTION_NONE,   QDP_STATE_BROAD_SRVRSP,
        /* TooMny */ QDP_ACTION_SLEEP,  QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_BROAD_RQSRV,
        /* NotReg */ QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   -1,                QDP_STATE_BROAD_RQSRV,
        /* Err    */ QDP_ACTION_REGERR, QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Ack    */ QDP_ACTION_RQPHY,  QDP_ACTION_NONE,   -1,                QDP_STATE_WAITPHY,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },
 
    { /* DataFSA state: waiting for physical interface data */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_RQCNF,  QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Hbeat  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_LDPHY,  QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_RQSRV,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_TLD,    QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_BROAD_RQSRV,
        /* Err    */ QDP_ACTION_CERR,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ack    */ QDP_ACTION_CACK,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_CTRL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_TLD,    QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },
    { /* DataFSA state: ready */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   -1,                QDP_STATE_RQSRV,
        /* CTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Hbeat  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,   
        /* Ack    */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* DataFSA state: server request sent */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Hbeat  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_BLDRSP, QDP_ACTION_SEND,   QDP_ACTION_NONE,   QDP_STATE_SRVRSP,
        /* TooMny */ QDP_ACTION_BUSY,   QDP_ACTION_NONE,   QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_REGERR, QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_REGERR, QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Ack    */ QDP_ACTION_RQCNF,  QDP_ACTION_NONE,   -1,                QDP_STATE_REGISTERED,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* DataFSA state: server challenge response sent */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_SEND,   QDP_ACTION_NONE,   -1,                QDP_STATE_SRVRSP,
        /* DTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Hbeat  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_BLDRSP, QDP_ACTION_SEND,   QDP_ACTION_NONE,   QDP_STATE_SRVRSP,
        /* TooMny */ QDP_ACTION_SLEEP,  QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_RQSRV,
        /* NotReg */ QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   -1,                QDP_STATE_RQSRV,
        /* Err    */ QDP_ACTION_REGERR, QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Ack    */ QDP_ACTION_RQCNF,  QDP_ACTION_NONE,   -1,                QDP_STATE_REGISTERED,
        /* Ctrl   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_FAIL,   QDP_ACTION_NONE,   -1,                QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* DataFSA state: registered */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_RQCNF,  QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Hbeat  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_LDCNF,  QDP_ACTION_RQMEM,  QDP_ACTION_NONE,   QDP_STATE_TOKEN,
        /* Token1 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_TLD,    QDP_ACTION_SRVRQ,  QDP_ACTION_NONE,   QDP_STATE_RQSRV,
        /* Err    */ QDP_ACTION_CERR,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ack    */ QDP_ACTION_CACK,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_CTRL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_TLD,    QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* DataFSA state: loading DP tokens */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_SEND,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Hbeat  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_LDCNF,  QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_LDMEM,  QDP_ACTION_RQMEM,  QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_LDMEM,  QDP_ACTION_TLU,    QDP_ACTION_OPEN,   QDP_STATE_READY,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_FLUSH,  QDP_ACTION_TLD,    QDP_ACTION_SRVRQ,  QDP_STATE_RQSRV,
        /* Err    */ QDP_ACTION_CERR,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ack    */ QDP_ACTION_CACK,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_CTRL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DATA,   QDP_ACTION_DACK,   QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_FILL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_USRCMD, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_TLD,    QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* DataFSA state: data link ready */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ -1,                -1,                -1,                QDP_STATE_OFF,
        /* CTO    */ QDP_ACTION_USRCMD, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ QDP_ACTION_DTO,    QDP_ACTION_DACK,   QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* Hbeat  */ QDP_ACTION_HBEAT,  QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* MySN   */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_LDCNF,  QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_CTRL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_CTRL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_FLUSH,  QDP_ACTION_TLD,    QDP_ACTION_SRVRQ,  QDP_STATE_RQSRV,
        /* Err    */ QDP_ACTION_CERR,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ack    */ QDP_ACTION_CACK,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_CTRL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_DATA,   QDP_ACTION_DACK,   QDP_ACTION_NONE,   QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_FILL,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_DROP,   QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_USRCMD, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_CRCERR, QDP_ACTION_NONE,   -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_TLD,    QDP_ACTION_FAIL,   QDP_ACTION_NONE,   QDP_STATE_OFF,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    },

    { /* DataFSA state: off */
        /* EVENT     ACTIONS                                                  NEXT STATE        */
        /* Up     */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* DTO    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Hbeat  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* MySN   */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* PhyRcv */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CnfRcv */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Token1 */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Token2 */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Srvch  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* TooMny */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* NotReg */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Err    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Ack    */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Ctrl   */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Data   */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Fill   */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Nosup  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Cmdrdy */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* CRCerr */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* IOerr  */ QDP_ACTION_NONE,   -1,                -1,                QDP_STATE_NOCHANGE,
        /* Off    */ QDP_ACTION_TLD,    QDP_ACTION_NONE,   -1,                QDP_STATE_OFF
    }
};

#endif /* INCLUDE_FSA_TABLE */

#ifdef __cplusplus
}
#endif

#endif /* qdp_fsa_included */

/*-----------------------------------------------------------------------+
 |                                                                       |
 | Copyright (C) 2006 Regents of the University of California            |
 |                                                                       |
 | This software is provided 'as-is', without any express or implied     |
 | warranty.  In no event will the authors be held liable for any        |
 | damages arising from the use of this software.                        |
 |                                                                       |
 | Permission is granted to anyone to use this software for any purpose, |
 | including commercial applications, and to alter it and redistribute   |
 | it freely, subject to the following restrictions:                     |
 |                                                                       |
 | 1. The origin of this software must not be misrepresented; you must   |
 |    not claim that you wrote the original software. If you use this    |
 |    software in a product, an acknowledgment in the product            |
 |    documentation of the contribution by Project IDA, UCSD would be    |
 |    appreciated but is not required.                                   |
 | 2. Altered source versions must be plainly marked as such, and must   |
 |    not be misrepresented as being the original software.              |
 | 3. This notice may not be removed or altered from any source          |
 |    distribution.                                                      |
 |                                                                       |
 +-----------------------------------------------------------------------*/

/* Revision History
 *
 * $Log: fsa.h,v $
 * Revision 1.13  2011/02/03 17:48:18  dechavez
 * added QDP_EVENT_DOWN, invoke QDP_ACTION_TLD in all cases
 *
 * Revision 1.12  2011/01/31 21:08:57  dechavez
 * added support for determining serial IP address
 *
 * Revision 1.11  2011/01/13 20:26:33  dechavez
 * added support for polling digitizer for serial number
 *
 * Revision 1.10  2010/03/22 21:30:17  dechavez
 * added QDP_ACTION_BUSY, changed TooMny servers response during handshake to invoke same
 *
 * Revision 1.9  2009/10/02 18:26:49  dechavez
 * Fixed the behavior in response to server busy during registration (it was
 * sending two registration requests... one right after waking up from the
 * sleep and one in response to the timeout.  Now we just wait for the timeout
 * for the next attempt.  The sleep is still there but could probably be
 * removed as well.
 *
 * Revision 1.8  2007/05/17 22:27:44  dechavez
 * initial production release
 *
 */
