#pragma ident "$Id: qio.h,v 1.4 2011/01/31 21:18:05 dechavez Exp $"
/*======================================================================
 *
 * Defines, templates, and prototypes Quanterra physical I/O library
 *
 *====================================================================*/
#ifndef qio_include_defined
#define qio_include_defined

#ifdef __cplusplus
extern "C" {
#endif

/* platform specific stuff */

#include "platform.h"
#include "util.h"
#include "ttyio.h"
#include "udpio.h"
#include "logio.h"

/* Handle for Digi I/O, here instead of its own library until the need arises. */

#define QIO_DIGI_PASSTHRU_PORT 2001

typedef struct {
    int sd;
} DIGIO;

/* Constants used in building the IP and UDP headers */

#define QIO_IP_VERSION  4
#define QIO_IP_HDRLEN  20
#define QIO_IP_DS       0
#define QIO_IP_TTL      0xff
#define QIO_IP_UDP      0x11
#define QIO_UDP_HDRLEN  8
#define QIO_IP_FLAGS    0x40
#define QIO_IP_FRAGOFF  0

/* I/O statistics */

typedef struct {
    MUTEX mutex;        /* for protection */
    struct {
        UINT64 bytes;   /* total number of bytes */
        UINT64 pkts;    /* total number of packets */
    } total;
    struct {
        int max;        /* largest packet */
        int min;        /* smallest packet */
    } len;
    UINT64 tstamp;      /* time of last I/O */
    struct {
        UINT64 count;   /* total number of errors */
        UINT64 tstamp;  /* time of last error */
    } error;
} QIO_STATS;

/* xmit and receive buffers */

#define QIO_BUFLEN 1500

typedef struct {
    UINT16 ident;          /* for unique idents in the IP header */
    QIO_STATS stats;       /* xmit statistics */
    UINT16 port;           /* source port number */
    UINT32 ip;             /* source IP address */
    UINT8 buf[QIO_BUFLEN]; /* xmit buffer */
} QIO_XMTBUF;

typedef struct {
    UTIL_TIMER timer;      /* for detecting input timeouts */
    UINT64 timeout;        /* timeout interval */
    int nread;             /* number of bytes in receive buffer */
    int index;             /* index to next unexamined byte in the buffer */
    QIO_STATS stats;       /* recv statistics */
    UINT8 buf[QIO_BUFLEN]; /* recv buffer */
} QIO_RCVBUF;

/* connection handle */

#define QIO_ERR -1
#define QIO_UDP  1 /* network */
#define QIO_TTY  2 /* serial port */
#define QIO_DIG  3 /* digi */

typedef struct {
    MUTEX mutex;
    int type;  /* QIO_UDP, QIO_TTY, QIO_DIG */
    union {
        UDPIO  udp; /* QIO_UDP */
        TTYIO *tty; /* QDP_TTY */
        DIGIO  dig; /* QDP_DIG */
    } method;
    QIO_XMTBUF xmit;
    QIO_RCVBUF recv;
    LOGIO *lp;
    char ident[MAXPATHLEN+1];
} QIO;

/* Default attributes for a serial connection */

#define QIO_READ_TO    25 /* read interval (msec) */
#define QIO_WRITE_TO 1000 /* write timeout interval (msec) */

#define QIO_DEFAULT_TTY_ATTR { \
    TRUE,                      \
    QIO_READ_TO,               \
    19200,                     \
    19200,                     \
    TTYIO_PARITY_NONE,         \
    TTYIO_FLOW_HARD,           \
    1,                         \
    0                          \
}

/* SLIP framing and escape characters */

#define QIO_FRM     0xC0
#define QIO_ESC     0xDB
#define QIO_ESC_FRM 0xDC
#define QIO_ESC_ESC 0xDD

/* close.c */
void qioClose(QIO *qio);

/* init.c */
int qioConnectionType(char *iostr, char *ident, TTYIO_ATTR *attr);
BOOL qioInit(QIO *qio, int port, char *iostr, int to, LOGIO *lp);

/* ip.c */
int qioPackIpHeader(UINT8 *start, QIO *qio, UINT32 dstip, int len);
BOOL qioVerifyIpHeader(UINT8 *start);

/* recv.c */
int qioRecv(QIO *qio, char *buf, int buflen);

/* send.c */
int qioSend(QIO *qio, UINT32 ip, int port, char *buf, int buflen);

/* stats.c */
void qioInitStats(QIO_STATS *stats);
void qioUpdateStats(QIO_STATS *stats, int nbytes, UINT64 tstamp);

/* udp.c */
int qioPackUdpHeader(UINT8 *start, QIO *qio, UINT8 *iphdr, int port, UINT8 *payload, int len);
BOOL qioVerifyUdpHeader(UINT8 *start, UINT8 *iphdr, UINT8 *payload, int len);

/* version.c */
VERSION *qioVersion(VOID);
char *qioVersionString(VOID);

#ifdef __cplusplus
}
#endif

#endif

/*-----------------------------------------------------------------------+
 |                                                                       |
 | Copyright (C) 2011 Regents of the University of California            |
 |                                                                       |
 | This software is provided 'as-is', without any express or implied     |
 | warranty.  In no event will the authors be held liable for any        |
 | damages arising from the use of this software.                        |
 |                                                                       |
 | Permission is granted to anyone to use this software for any purpose, |
 | including commercial applications, and to alter it and redistribute   |
 | it freely, subject to the following restrictions:                     |
 |                                                                       |
 | 1. The origin of this software must not be misrepresented; you must   |
 |    not claim that you wrote the original software. If you use this    |
 |    software in a product, an acknowledgment in the product            |
 |    documentation of the contribution by Project IDA, UCSD would be    |
 |    appreciated but is not required.                                   |
 | 2. Altered source versions must be plainly marked as such, and must   |
 |    not be misrepresented as being the original software.              |
 | 3. This notice may not be removed or altered from any source          |
 |    distribution.                                                      |
 |                                                                       |
 +-----------------------------------------------------------------------*/

/* Revision History
 *
 * $Log: qio.h,v $
 * Revision 1.4  2011/01/31 21:18:05  dechavez
 * changed read timeout to 25 ms
 *
 * Revision 1.3  2011/01/31 21:10:31  dechavez
 * initial release (only tty and UDP I/O tested)
 *
 */
