#pragma ident "$Id: slink.c,v 1.11 2013/03/08 19:56:01 dechavez Exp $"
/*======================================================================
 *
 * Tee incoming ISI data into a SeedLink server.
 *
 * Uses Chad Trabant's libdali and libmseed, lifted from orb2ringserver
 *
 *====================================================================*/
#include "isi/dl.h"
#include "nrts/dl.h"
#include "dmc/libdali.h"
#include "dmc/libmseed.h"

static LOGIO *lp = NULL; /* this is wrong, but I'm not worrying about it */

static BOOL QueueIsEmpty(ISI_DL_SEEDLINK *slink)
{
BOOL retval;

    MUTEX_LOCK(&slink->queue.mutex);
        retval = slink->queue.empty;
    MUTEX_UNLOCK(&slink->queue.mutex);

    return retval;
}

static char DescToMseedSampletype(ISI_DATA_DESC *desc)
{
    switch (desc->type) {
      case ISI_TYPE_INT32:  return 'i'; break;
      case ISI_TYPE_REAL32: return 'f'; break;
      case ISI_TYPE_REAL64: return 'd'; break;
    }

    return 0;
}

static void EnqueueMiniSEED(ISI_DL_SEEDLINK *slink, UINT8 *record)
{
static char *fid = "EnqueueMiniSEED";
     
    MUTEX_LOCK(&slink->queue.mutex);
     
        if (listCount(slink->queue.list) == slink->queue.depth) {
            if (++slink->queue.dropped == 1) {
                logioMsg(slink->lp, LOG_INFO, "*** WARNING *** %s: queue is full (%d packets), MiniSEED packet dropped",
                    fid, slink->queue.depth
                );
            }
        } else {
            if (!listAppend(slink->queue.list, record, slink->mseed.reclen)) {
                logioMsg(slink->lp, LOG_INFO, "*** ERROR *** %s: listAppend: %s", fid, strerror(errno));
                logioMsg(slink->lp, LOG_INFO, "*** ERROR *** %s: SeedLink export terminated", fid);
                slink->enabled = FALSE;
            }
            SEM_POST(&slink->queue.sem);
        }
        slink->queue.empty = FALSE; 

    MUTEX_UNLOCK(&slink->queue.mutex);
}

static BOOL DequeueMiniSEED(ISI_DL_SEEDLINK *slink, UINT8 *record)
{
BOOL retval;
LNKLST_NODE *crnt;
static char *fid = "DequeueMiniSEED";

    MUTEX_LOCK(&slink->queue.mutex);
     
        crnt = listFirstNode(slink->queue.list);
        if (crnt != NULL) {
            memcpy(record, (UINT8 *) crnt->payload, slink->mseed.reclen);
            listDestroyNode(crnt);
            retval = TRUE;
        } else {
            retval = FALSE;
            slink->queue.empty = TRUE;
            slink->queue.dropped = 0;
        }
     
    MUTEX_UNLOCK(&slink->queue.mutex);

    return retval;
}

static void SeedLinkFullRecordCallback(char *record, int reclen, void *argptr)
{
ISI_DL_SEEDLINK *slink;
static char *fid = "SeedLinkFullRecordCallback";

    EnqueueMiniSEED((ISI_DL_SEEDLINK *) argptr, (UINT8 *) record);
}

static void ConnectToRingServer(ISI_DL_SEEDLINK *slink)
{
DLCP *dlcp;

    dlcp = (DLCP *) slink->dlcp;

    if (dlcp->link > 0) return;

    while (1) {
        if (dl_connect(dlcp) != -1) {
            logioMsg(slink->lp, LOG_INFO, "connected to SeedLink@%s", slink->rsaddr);
            return;
        }
        utilDelayMsec(slink->retry);
    }
}

static void SendToRingServer(ISI_DL_SEEDLINK *slink, char *record)
{
DLCP *dlcp;
MSRecord *msr = NULL;
char streamid[100];
hptime_t endtime;
int writeack = 0;
int rv;
static char *fid = "SendToRingServer";

    dlcp = (DLCP *) slink->dlcp;

    if ((rv = msr_unpack(record, slink->mseed.reclen, &msr, 0, 0)) != MS_NOERROR) {
        ms_recsrcname(record, streamid, 0);
        logioMsg(slink->lp, LOG_ERR, "%s: msr_unpack: %s: %s", fid, streamid, ms_errorstr(rv));
        return;
    }

    msr_srcname(msr, streamid, 0);
    strcat(streamid, "/MSEED");
    endtime = msr_endtime(msr);

    while (dl_write(dlcp, record, slink->mseed.reclen, streamid, msr->starttime, endtime, writeack) < 0) {
        logioMsg(slink->lp, LOG_WARN, "%s: dl_write failed", fid);
        dl_disconnect(dlcp);
        ConnectToRingServer(slink);
    }

    msr_free(&msr);
}

/* Main loop.  Take full MiniSEED packets from the queue and send to the ringserver */

static THREAD_FUNC RingServerWriteThread(void *argptr)
{
ISI_DL_SEEDLINK *slink;
char *record;
static char *fid = "RingServerWriteThread";

    slink = (ISI_DL_SEEDLINK *) argptr;
    SEM_POST(&slink->sem);

    if ((record = (UINT8 *) malloc(slink->mseed.reclen)) == NULL) {
        logioMsg(slink->lp, LOG_INFO, "*** ERROR *** %s: malloc: %s", fid, strerror(errno));
        logioMsg(slink->lp, LOG_INFO, "** NOTICE ** SeedLink feed terminated");
        THREAD_EXIT((void *) 0);
    }

    ConnectToRingServer(slink);

    while (1) {
        SEM_WAIT(&slink->queue.sem);
        while (DequeueMiniSEED(slink, record)) SendToRingServer(slink, record);
    }
}

static BOOL ParseServerString(char *string, ISI_DL_SEEDLINK *slink)
{
#define NUM_TOKEN 5
int ntoken, port;
char *token[NUM_TOKEN], *copy;
static char *fid = "ParseServerString";

    if (string == NULL) {
        logioMsg(slink->lp, LOG_ERR, "%s: NULL config string not allowed\n", fid);
        errno = EINVAL;
        return FALSE;
    }

    if ((copy = strdup(string)) == NULL) {
        logioMsg(slink->lp, LOG_ERR, "%s: strdup: %s", fid, strerror(errno));
        return FALSE;
    }

/* expecting strings of the form server:port:len:depth:net, eg "localhost:16000:512:500:II" */

    if ((ntoken = utilParse(copy, token, ":", NUM_TOKEN, 0)) < 0) {
        logioMsg(slink->lp, LOG_ERR, "%s: utilParse: %s\n", fid, strerror(errno));
        return FALSE;
    }
    if (ntoken != NUM_TOKEN) {
        logioMsg(slink->lp, LOG_ERR, "%s: illegal slink parameter string '%s'\n", fid, string);
        errno = EINVAL;
        return FALSE;
    }

/* 1st token is the name/IP of the server */

    strncpy(slink->rsaddr, token[0], MAXPATHLEN);

/* 2nd token is the port */

    if ((port = atoi(token[1])) < 1) {
        logioMsg(slink->lp, LOG_ERR, "%s: illegal port value '%s'\n", fid, token[1]);
        errno = EINVAL;
        return FALSE;
    }
    strcat(slink->rsaddr, ":");
    strcat(slink->rsaddr, token[1]);

/* 3rd token is the record length (must be a power of 2) */

    slink->mseed.reclen = atoi(token[2]);
    if (util_powerof(slink->mseed.reclen, 2) < 0) {
        logioMsg(slink->lp, LOG_ERR, "%s: illegal reclen value '%s' (not a power of 2)\n", fid, token[2]);
        errno = EINVAL;
        return FALSE;
    }

/* 4th token is the queue depth */

    if ((slink->queue.depth = (UINT32) atoi(token[3])) < 1) {
        logioMsg(slink->lp, LOG_ERR, "%s: illegal depth value '%s'\n", fid, token[3]);
        errno = EINVAL;
        return FALSE;
    }

/* 5th and final token is the network code */

    strncpy(slink->mseed.net, token[4], 2);

    return TRUE;
}

/* Intercept the libdali and libmseed log messages */

static void my_log_print(char *string)
{
    logioMsg(lp, LOG_INFO, string);
}

static void my_diag_print(char *string)
{
    logioMsg(lp, LOG_ERR, string);
}

void isidlLogSeedLinkOption(LOGIO *logio, int level, ISI_DL *dl)
{
    if (dl == NULL) return;

    if (!dl->slink.enabled) {
        logioMsg(logio, level, "SeedLink export disabled");
        return;
    }

    logioMsg(logio, level, "SeedLink export enabled: %s:%d:%d:%s",
        dl->slink.rsaddr, dl->slink.mseed.reclen, dl->slink.queue.depth, dl->slink.mseed.net
    );
}

BOOL isidlSetSeedLinkOption(ISI_DL *dl, char *argstr, char *ident)
{
LNKLST *tlist;
char rsaddr[MAXPATHLEN+1];
static char *fid = "isidlSetSeedLinkOption";

    if (dl == NULL || argstr == NULL) {
        errno = EINVAL;
        return FALSE;
    }

    if (dl->slink.enabled) {
        logioMsg(dl->lp, LOG_ERR, "%s: multiple SeedLink servers not allowed\n");
        errno = EINVAL;
        return FALSE;
    }

    dl->slink.lp = dl->lp;

    if (!ParseServerString(argstr, &dl->slink)) return FALSE;

    if ((dl->slink.queue.list = listCreate()) == NULL) {
        logioMsg(dl->slink.lp, LOG_ERR, "%s: listInit: %s", fid, strerror(errno));
        return FALSE;
    }
    MUTEX_INIT(&dl->slink.queue.mutex);
    SEM_INIT(&dl->slink.queue.sem, 0, 1);
    dl->slink.queue.empty = TRUE;
    dl->slink.queue.dropped = 0;

    if ((dl->slink.dlcp = (void *) dl_newdlcp(dl->slink.rsaddr, ident)) == 0) {
        logioMsg(dl->slink.lp, LOG_ERR, "%s: dl_newdlcp: %s", fid, strerror(errno));
        return FALSE;
    }

    dl_loginit(0, (void *) my_log_print, NULL, (void *) my_diag_print, NULL);
    ms_loginit(   (void *) my_log_print, NULL, (void *) my_diag_print, NULL);

    dl->slink.mseed.msr = NULL;
    if ((dl->slink.mseed.mstg = mst_initgroup(NULL)) == NULL) {
        logioMsg(dl->slink.lp, LOG_ERR, "%s: mst_initgroup: %s", fid, strerror(errno));
        return FALSE;
    }
    dl->slink.retry   = ISI_DL_DEFAULT_SLINK_RETRY; /* no option to change */
    dl->slink.first   = TRUE; /* thread gets launched on receipt of first packet */
    dl->slink.enabled = TRUE;

    return TRUE;
}

static BOOL PackIntoMiniSEED(ISI_DL_SEEDLINK *slink, MSTrace *mst, int flush)
{
int encoding;
static char *fid = "isidlFeedSeedLink:PackIntoMiniSEED";

    if (mst->numsamples <= 0) return TRUE;
    if (!slink->enabled) return FALSE;

    switch (mst->sampletype) {
      case 'i': encoding = DE_STEIM2;  break;
      case 'f': encoding = DE_FLOAT32; break;
      case 'd': encoding = DE_FLOAT64; break;
      default:
        logioMsg(slink->lp, LOG_ERR, "*** ERROR *** %s: unsupported sample type '%c'", fid, mst->sampletype);
        logioMsg(slink->lp, LOG_WARN, "** NOTICE ** SeedLink feed terminated");
        slink->enabled = FALSE;
        return FALSE;
    }

    if (mst_pack(mst, SeedLinkFullRecordCallback, (void *) slink, slink->mseed.reclen, encoding, 1, NULL, flush, 0, NULL) < 0) {
        logioMsg(slink->lp, LOG_ERR, "*** ERROR *** %s: mst_pack failed", fid);
        logioMsg(slink->lp, LOG_WARN, "** NOTICE ** SeedLink feed terminated");
        slink->enabled = FALSE;
        return FALSE;
    }

    return TRUE;
}

void isidlFeedSeedLink(ISI_DL_SEEDLINK *slink, ISI_RAW_PACKET *raw, NRTS_PKT *pkt)
{
time_t QuitTime;
ISI_GENERIC_TS *gen;
MSTrace *mst;
MSRecord *msr;
MSTraceGroup *mstg = NULL;
static char *fid = "isidlFeedSeedLink";

    if (slink == NULL || !slink->enabled) return;

/* NULL packet means flush all incomplete MiniSEED packets */

    if (raw == NULL) {
        logioMsg(slink->lp, LOG_INFO, "flushing all partial SeedLink packets");
        mstg = (MSTraceGroup *) slink->mseed.mstg;
        mst = mstg->traces;
        while (mst) {
            if (!PackIntoMiniSEED(slink, mst, 1)) return;
            mst = mst->next;    
        }
        /* give RingServerWriteThread time to drain the queue */
        QuitTime = time(NULL) + 10; /* 10 seconds max */
        while (!QueueIsEmpty(slink) && time(NULL) < QuitTime) sleep(1);
        return;
    }

/* First time through, launch the packet forwarding thread */

    if (slink->first) {
        SEM_INIT(&slink->sem, 0, 1);
        if (!THREAD_CREATE(&slink->tid, RingServerWriteThread, (void *) slink)) {
            logioMsg(slink->lp, LOG_ERR, "*** ERROR *** %s: can't start RingServerWriteThread!", fid);
            logioMsg(slink->lp, LOG_WARN, "** NOTICE ** SeedLink feed terminated");
            slink->enabled = FALSE;
            return;
        }
        SEM_WAIT(&slink->sem);
        slink->first = FALSE;
    }

/* If this is already a properly sized MiniSEED record, just send it on */

    if (raw->hdr.desc.type == ISI_TYPE_MSEED && raw->hdr.len.used == slink->mseed.reclen) {
        EnqueueMiniSEED(slink, raw->payload);
        return;
    }

/* If this is not a supported packet type, or if it has problems, then ignore it */

    if (pkt == NULL || pkt->status != NRTS_ACCEPT) return;
    gen = &pkt->isi;  /* for readability */

/* At this point we have waveform data in generic form */

    if (!(slink->mseed.msr = msr_init(slink->mseed.msr))) {
        logioMsg(slink->lp, LOG_ERR, "*** ERROR *** %s: msr_init: ", fid, strerror(errno));
        logioMsg(slink->lp, LOG_WARN, "** NOTICE ** SeedLink feed terminated");
        slink->enabled = FALSE;
        return;
    }

/* We do the assignments below just to make the code easier to read */

    msr = (MSRecord *) slink->mseed.msr;
    mstg = (MSTraceGroup *) slink->mseed.mstg;

/* Populate MSRecord */

    ms_strncpclean(msr->network,  slink->mseed.net,  2); util_ucase(msr->network);
    ms_strncpclean(msr->station,  gen->hdr.name.sta, 5); util_ucase(msr->station);
    ms_strncpclean(msr->location, gen->hdr.name.loc, 2); util_ucase(msr->location);
    ms_strncpclean(msr->channel,  gen->hdr.name.chn, 3); util_ucase(msr->channel);

    msr->starttime   = (hptime_t) (gen->hdr.tofs.value * HPTMODULUS);
    msr->samprate    = gen->hdr.srate.value;
    msr->datasamples = gen->data;
    msr->numsamples  = gen->hdr.nsamp;
    msr->samplecnt   = gen->hdr.nsamp;
    if ((msr->sampletype  = DescToMseedSampletype(&gen->hdr.desc)) == 0) return; /* 0 means unsupported type */

/* Add data to trace buffer, creating new entry or extending as needed */

    if (!(mst = mst_addmsrtogroup(mstg, msr, -1, -1.0, -1.0))) {
        logioMsg(slink->lp, LOG_ERR, "*** ERROR *** %s: mst_addmsrtogroup: %s", fid, strerror(errno));
        logioMsg(slink->lp, LOG_WARN, "** NOTICE ** SeedLink feed terminated");
        slink->enabled = FALSE;
        return;
    }

/* Pack into MiniSEED */

    PackIntoMiniSEED(slink, mst, 0); /* 0 means don't flush until packets are full */
}

/* Revision History
 *
 * $Log: slink.c,v $
 * Revision 1.11  2013/03/08 19:56:01  dechavez
 * fixed bug with timeout logic during queue flush
 *
 * Revision 1.10  2013/03/08 18:29:54  dechavez
 * added ability to flush partial MiniSEED packets
 *
 * Revision 1.9  2013/02/07 23:33:12  dechavez
 * Memory leak was NOT fixed in 1.21.1 (sigh).  Reworked buffering. Again.
 *
 * Revision 1.8  2013/02/07 21:11:52  dechavez
 * fixed LNKLST_MODE memory leak in PopMiniSEED
 *
 * Revision 1.7  2013/02/07 18:12:47  dechavez
 * reworked to use RECORDQ for buffering output data
 *
 * Revision 1.6  2013/02/05 21:23:15  dechavez
 * fixed time stamp truncation error (MAJOR BUG!)
 *
 * Revision 1.5  2013/01/23 23:48:44  dechavez
 * init slink->mseed.msr to NULL, instead of using msr_init()
 *
 * Revision 1.4  2013/01/23 23:35:01  dechavez
 * fixed uninitialized slink->mseed.msr BUG, added extra sanity checks, improved(?) logging
 *
 * Revision 1.3  2013/01/22 22:32:17  dechavez
 * don't print message when tossing full MiniSEED packets after queue buffer is full
 *
 * Revision 1.2  2013/01/22 21:26:51  dechavez
 * fixed memory link in SendToRingServer(), set msgq log pointer
 *
 * Revision 1.1  2013/01/14 22:12:20  dechavez
 * created
 *
 */
