#pragma ident "$Id: mini.c,v 1.4 2012/02/14 20:22:07 dechavez Exp $"
/*======================================================================
 *
 *  Compress an array of INT32s (host byte order) into mini-SEED packets
 *  of user specified length.  The function will compress as many words
 *  as it can before the output buffer is filled (including the fixed
 *  section of data header and the blockette 1000), and returns the
 *  number of samples compressed.  The calling program should deal with
 *  the full output packet, update the input pointer and number of
 *  samples to reflect the amount of processed data and repeat, until
 *  such time that the entire array is processed.
 *
 *  Assumes nominal sample rates (ie, no blockette 100).
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Copyright (c) 1997 Regents of the University of California.
 * All rights reserved.
 *====================================================================*/
#include <sys/types.h>
#include <math.h>
#include "seed.h"
#include "util.h"

INT32 seed_mini(INT32 *input, INT32 count, struct seed_fsdh *fsdh, UINT8 *output, int outexp, INT32 *used)
{
INT32 nc, outlen;
struct seed_b1000 b1000;
static char *fid = "seed_mini";

/* Determine actual record length and clear the buffer */

    if (outexp < SEED_MINRECEXP || outexp > SEED_MAXRECEXP) {
        util_log(1, "%s: illegal outexp (%d)", fid, outexp);
        return -1;
    }
        
    outlen = (INT32) pow(2.0, (double) outexp);
    b1000.length = (u_char) outexp;

    memset(output, 0, outlen);

/* Since we don't have blockette 100, Steim 1 data start at offset 64 */

    fsdh->bod   = 64;
    fsdh->first = 48;
    nc = util_csteim1(output+fsdh->bod,outlen-fsdh->bod,input,count,used);
    *used += fsdh->bod;

/* Set end of time series flag if appropriate */

    if (nc == count) {
        fsdh->ioclck |= SEED_FLAG_STOP;
        util_log(2, "set STOP flag");
    }

/* Load the FSDH and blockette 1000 in the front of the buffer */

    fsdh->nsamp = (INT16) nc;
    fsdh->more  = 1; /* just a blockette 1000 to follow */
    seed_loadfsdh(output, fsdh, BIG_ENDIAN_ORDER);

    b1000.next   = 0;
    b1000.format = SEED_STEIM1;
    b1000.order  = 1; /* BIG_ENDIAN_ORDER */
    seed_load1000(output+fsdh->first, &b1000, BIG_ENDIAN_ORDER);

/* Return the number of words compressed */

    return nc;
}

/* Revision History
 *
 * $Log: mini.c,v $
 * Revision 1.4  2012/02/14 20:22:07  dechavez
 * use portable integer types (eg, long to INT32, unsigned long to UINT32)
 *
 * Revision 1.3  2007/01/11 17:48:03  dechavez
 * added platform.h and stdtypes.h stuff
 *
 * Revision 1.2  2005/05/25 22:40:22  dechavez
 * mods to calm Visual C++ warnings
 *
 * Revision 1.1.1.1  2000/02/08 20:20:39  dec
 * import existing IDA/NRTS sources
 *
 */
