/************************************************************************/
/*  ms2csv -	Output MiniSEED data in CSV format.			*/
/*									*/
/*	Douglas Neuhauser						*/
/*	Berkeley Seismological Laboratory				*/
/*	University of California, Berkeley				*/
/*	doug@seismo.berkeley.edu					*/
/*									*/
/************************************************************************/

/*
 * Copyright (c) 2012 The Regents of the University of California.
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes,
 * without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice, this paragraph and the
 * following three paragraphs appear in all copies.
 * 
 * Permission to incorporate this software into commercial products may
 * be obtained from the Office of Technology Licensing, 2150 Shattuck
 * Avenue, Suite 510, Berkeley, CA  94704.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND
 * ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
 * CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#ifndef lint
static const char sccsid[] = "$Id: ms2csv.c,v 1.3 2013/09/17 18:35:51 doug Exp $ ";
#endif

#include <stdio.h>
#include "qlib2.h"
#include "version.h"

#include "externals.h"

char *syntax[] = {
"%s version " VERSION,
"%s - Output MiniSEED data in CSV format",
"%s [-D j|m] [-d delim] [-h] [input_file] [output_file]",
"    where:",
"	-d delim    Specify CSV delimiter character. Default is ',' (comma).",
"	-D j | m    Display dates in julian or month and day format.",
"	-p 1..6     Output seconds with n digits of precision after the",
"		    decimal point.  4 = std MiniSEED, 6 = use blockette 1001.",
"		    Default is 6.  Uses truncation, not rounding.",
"	-h	    Help - prints syntax message.",
"	input_file",
"		    Data file with original data.  If no file is specified,",
"		    read from stdin.",
"	output-file",
"		    Data written to specified file.  If no file is specified,",
"		    write to stdout.",
NULL };

#include <stdlib.h>
#include <string.h>
#include <math.h>

int parse_cmdline (int *pargc, char ***pargv);

/************************************************************************/
/*  main:   main program						*/
/************************************************************************/
int main (int argc, char **argv)
{
    int i;
    FILE *fpin, *fpout;
    DATA_HDR *hdr = NULL;
    char *mseed = NULL;
    int blksize;
    int rc = 0;
    int divisor[7] = { 1000000, 100000, 10000, 1000, 100, 10, 1 };

    /*	Initialization.							*/
    output_file= NULL;		/* empty initial output directory.	*/
    fpin = stdin;		/* assume input to come from stdin.	*/
    info = stderr;
    date_fmt = JULIAN_FMT;

    parse_cmdline (&argc, &argv);

    if (input_file) {
	if ((fpin = fopen (input_file,"r")) == NULL) {
	    fprintf (info, "Unable to open input file %s\n", input_file);
	    exit(1);
	}
    }
    else {
	input_file = "<stdin>";
	fpin = stdin;
    }

    if ((output_file && strcmp(output_file, "-") != 0)) {
	if ((fpout = fopen (output_file,"w")) == NULL) {
	    fprintf (info, "Unable to open output file %s\n", output_file);
	    exit(1);
	}
    }
    else {
	info = stderr;
	output_file = "<stdout>";
	fpout = stdout;
    }


    int *isamples = NULL;
    float *fsamples = NULL;
    double *dsamples = NULL;
    int maxsamples = 0;
    int first_record = 1;
    DATA_HDR *fhdr = NULL;
    while (( hdr=NULL, mseed=NULL, blksize = read_ms_record (&hdr, &mseed, fpin)) > 0) {
	int nsamples;
	INT_TIME bt;
	BS *bs;			/* ptr to blockette structure.		*/
	BLOCKETTE_1000 *b1000;	/* ptr to blockette 1000.		*/

	if (maxsamples < hdr->num_samples) {
	    if (dsamples) free(dsamples);
	    dsamples = NULL;
	    dsamples = malloc (sizeof(double) * hdr->num_samples);
	    isamples = (int *)dsamples;
	    fsamples = (float *)dsamples;
	    if (dsamples) maxsamples = hdr->num_samples;
	    else {
		fprintf (info, "Error mallocing space for %d samples\n", hdr->num_samples);
		fflush (fpout);
		exit(1);
	    }
	}

	if ((nsamples = ms_unpack (hdr, hdr->num_samples, mseed, dsamples)) < 0) {
	    fprintf (info, "Error %d unpacking MiniSEED data\n", nsamples);
	    rc = nsamples;
	    break;
	}

	if (nsamples == 0) continue;

	if (first_record) {
	    first_record = 0;
	    fprintf (fpout, "#S.N.C.L%c%s.%s.%s.%s\n", delim, 
		    hdr->station_id, hdr->network_id, hdr->channel_id, hdr->location_id);
	    if (date_fmt == JULIAN_FMT || date_fmt == JULIAN_FMT_1) {
		fprintf (fpout, "#year%cdoy%chour%cminute%csecond%cvalue\n",delim,delim,delim,delim,delim);
	    }
	    else {
		fprintf (fpout, "#year%cmon%cday%chour%cminute%csecond%cvalue\n",delim,delim,delim,delim,delim,delim);
	    }
	    fhdr = dup_data_hdr(hdr);
	}

	/* Ensure that all data is for the same SNCL. */
	if (strcmp(fhdr->station_id, hdr->station_id) ||
	    strcmp(fhdr->network_id, hdr->network_id) ||
	    strcmp(fhdr->channel_id, hdr->channel_id) ||
	    strcmp(fhdr->location_id, hdr->location_id)) {
	    fprintf (info, "Error: expecting SNCL %s.%s.%s.%s, found %s.%s.%s.%s\n",
		     hdr->station_id, hdr->network_id, hdr->channel_id, hdr->location_id,
		     fhdr->station_id, fhdr->network_id, fhdr->channel_id, fhdr->location_id);
	    rc = 1;
	    break;
	}
	/* Determine blocksize and data format from the blockette 1000.	*/

	/* If we don't have one, it is an error.				*/
	if ((bs = find_blockette (hdr, 1000)) == NULL) {
	    rc = MS_ERROR;
	    break;
	}
	b1000 = (BLOCKETTE_1000 *)bs->pb;
	int format = b1000->format;

	bt = hdr->begtime;
	for (i=0; i<nsamples; i++) {
	    int iseconds, iuseconds, irusec;
	    double dseconds;
	    INT_TIME it;
	    EXT_TIME et;
	    time_interval2 (i, hdr->sample_rate, hdr->sample_rate_mult, &iseconds, &iuseconds);
	    it = add_time (bt, iseconds, iuseconds);
	    et = int_to_ext(it);
/*	    dseconds = et.second + ((double)et.usec/USECS_PER_SEC); */
	    irusec = ((int)(et.usec/divisor[time_precision]))*divisor[time_precision];
	    dseconds = et.second + ((double)irusec/USECS_PER_SEC);

		
	    if (date_fmt == JULIAN_FMT || date_fmt == JULIAN_FMT_1) {
		fprintf (fpout, "%d%c%d%c%d%c%d%c%.*lf%c",
			 et.year, delim, et.doy, delim, 
			 et.hour, delim, et.minute, delim,
			 time_precision, dseconds, delim);
	    }
	    else {
		fprintf (fpout, "%d%c%d%c%d%c%d%c%d%c%.*lf%c",
			 et.year, delim, et.month, delim, et.day, delim, 
			 et.hour, delim, et.minute, delim, 
			 time_precision, dseconds, delim);
	    }
	    switch (mseed_to_data_format(format)) {
	    case DATA_FMT_INT:
		fprintf (fpout, "%d\n", isamples[i]);
		break;
	    case DATA_FMT_FLOAT:
		fprintf (fpout, "%f\n", fsamples[i]);
		break;
	    case DATA_FMT_DOUBLE:
		fprintf (fpout, "%lf\n", dsamples[i]);
		break;
	    default:
		fprintf (fpout, "\nError - unknown MiniSEED format: %d\n", format);
		rc = 1;
		break;
	    }
	    if (rc) break;
	}

	if (hdr) free_data_hdr(hdr);
	if (mseed) free(mseed);
	if (rc) break;
    }

    /* Cleanup before exit. */
    if (hdr) free_data_hdr(hdr);
    if (mseed) free (mseed);
    fclose (fpin);
    if (fpout) fclose (fpout);
    return(rc);
}
