#!/usr/bin/env perl

use strict;
use warnings;
use Getopt::Std;
use File::Basename;
use IO::Handle;
use Data::Dumper;

# global vars
use vars qw ( $opt_h $opt_f $opt_t $opt_m $opt_d $opt_w $opt_r $opt_o $opt_e $opt_a $opt_O $opt_s );
use vars qw ( $version $prog );

# these directories will need to be set accordingly
my $WORK_DIR = "/work/safod1/dcmgr/strain/tmp$$";
my $RAW_DIR1 = "/data/sfraw2/strain/raw_data/femto";
my $RAW_DIR2 = "/data/sfraw2/strain/raw_data/athene";
my $RAW_DIR3 = "/data/sfraw2/strain/raw_data/pfemto";
my $DEST_DIR = "/work/dc11/dcmgr/sfvsm";
my $INSTALL_DIR = "/data/seismo39/dcmgr/src";
my $TEST_DIR = "${INSTALL_DIR}/tee2ms/test";
my $QLIB2_DIR = "/data/seismo39/dcmgr/src/qlib2";
my $TEST_CHAN = "MHVSM.SF.DSZ.D2";
my $OS = `uname -s`; chomp $OS;
my $MC = `uname -p`; chomp $MC;
my $TEE2MS_PROD = "${INSTALL_DIR}/tee2ms/bin/tee2ms";
my $MS2CSV = "${INSTALL_DIR}/ms2csv/ms2csv";
my $TEE2MS_TEST = "../../tee2ms";
my $EXCEPTIONS = "${INSTALL_DIR}/tee2ms/exceptions";
my $TIME_TOL = 0.00031;
my $VALUE_TOL = 0.000001;
my %exceptions = ();

sub help {
#
# command line arguments
#

    printf "Convert SAFOD stain data to mSEED\n
      $0 [-h] -f <doy> -t <doy> [-m chan.map], where
      -h          print help message
      -f <doy>    from <doy> where <doy> is yyyy.ddd
      -t <doy>    to <doy> where <doy> is yyyy.ddd
      -m <file>   channel map file, default is 'chan.map'
      -d          turn on debugging
      -w          turn off warnings; won't work if debug is on
      -r          run regression tests
      -o <offset> apply offset <offset> tics (1 tic = 10us)
      -e <file>   exceptions file, default is 'exceptions'
      -O <dir>    full path name of output directory
      -a          convert all existing data to mSEED
      -s          just print stats for specified data range
     \n";

    exit(0);
}


sub sys_cmd {
#
# execute UNIX shell command
#
    my ($cmd) = @_;
    my ($rc) = 0xffff & system($cmd);

    warn " $cmd\n" if ($opt_d);
    if($rc != 0) {
	print "Error: System command $cmd failed\n" if ($opt_d);
	return 1;
    }

    return 0
}


sub prev_ydoy {
#
# return previous doy as yyyy.ddd
#
    my ($ydoy) = @_;
    my $new_ydoy;

    if( $OS eq "Linux" ) {
	$new_ydoy = `caldate -f %Y.%j $ydoy -- -1d`;
    } else {
	$new_ydoy = `caldate -f %Y.%j $ydoy -1d`;
    }
    chomp $new_ydoy;
      
    return $new_ydoy;
}


sub next_ydoy {
#
# return next doy as yyyy.ddd
#
    my ($ydoy) = @_;
    my $new_ydoy = `caldate -f %Y.%j $ydoy +1d`; chomp $new_ydoy;
      
    return $new_ydoy;
}


sub diff_ydoy {
#
# difference in secs between 2 yyyy.ddd
#
    my ($ydoy1, $ydoy2) = @_;
    my $diff = `caldate -s $ydoy1 $ydoy2`; chomp $diff;
      
    return $diff;
}


sub do_tee2ms {
    my ($prog, $map_file, $out_dir, $yyyy, $ddd) = @_;
    my $cmd;
    my $options="";
    my $ydoy = "${yyyy}.${ddd}";

    # apply external offset if specified in exceptions file and/or command line
    $options = $exceptions{$ydoy} if ( defined $exceptions{$ydoy} && $exceptions{$ydoy} !~ /missing|discard/ );

    # Prefer FEMTO over ATHENE data
    if( -e "$RAW_DIR1/$yyyy/$ddd/sfodi1" ) {
	$cmd = "${prog} -w -T 1 -z -b 512 -o $out_dir -m $map_file $options ${RAW_DIR1}/${yyyy}/${ddd}/sfodi1 > ${yyyy}.${ddd}_tee2ms.log 2>&1";
    } else {
	if ( -d "${RAW_DIR2}/${yyyy}_${ddd}" ) {
	    $cmd = "${prog} -w -T 1 -b 512 -o $out_dir -m $map_file $options ${RAW_DIR2}/${yyyy}_${ddd}/\\*.gz > ${yyyy}.${ddd}_tee2ms.log 2>&1";
	} else {
	    if ( -d "${RAW_DIR3}/${yyyy}/${ddd}" ) {
		 $cmd = "${prog} -w -T 1 -b 512 -o $out_dir -m $map_file $options ${RAW_DIR3}/${yyyy}/${ddd}/\\*.gz > ${yyyy}.${ddd}_tee2ms.log 2>&1";
	    } else {
		warn " Warning: No input data file for ${yyyy}.${ddd}" if (!$opt_w);
		return 1;
	    }
	}
    }
    return 1 if (&sys_cmd($cmd));

    # rename rough daily mSEED output files for easy identification
    foreach my $chan ( `grep -v "#" $map_file | awk '{print \$2}'` ) {
	chomp $chan;
	next if ( ! -d "$out_dir/$chan" );
	if ( `find $out_dir/${chan} -name ${chan}.?????????????? | wc -l` > 0 ) {
	    $cmd = "/bin/mv -f $out_dir/${chan}/${chan}.?????????????? $out_dir/${chan}/${chan}.${yyyy}.${ddd}";
	    return 1 if(&sys_cmd($cmd));
	}
	if ( -e "$out_dir/${chan}/${chan}.${yyyy}.${ddd}" && -z "$out_dir/${chan}/${chan}.${yyyy}.${ddd}" ) {
	    $cmd = "/bin/rm -f $out_dir/${chan}/${chan}.${yyyy}.${ddd}";
	    return 1 if(&sys_cmd($cmd));
	}
    }

    # clean up
    if ( -e "${yyyy}.${ddd}_tee2ms.log" ) {
	$cmd = "/bin/rm -f ${yyyy}.${ddd}_tee2ms.log";
	return 1 if(&sys_cmd($cmd));
    }

    return 0;
}


sub do_convert {
#
# convert doy to mSEED
#
    my ($ydoy) = @_;
    my ($yyyy, $ddd) = split /\./, $ydoy;
    my ($cmd, $rc);

    # don't bother with known missing or bad files
    return 1 if ( defined $exceptions{$ydoy} && ($exceptions{$ydoy} eq "missing" || $exceptions{$ydoy} eq "discard" ) );

    # clean up files
    foreach my $chan ( `grep -v "#" $opt_m | awk '{print \$2}'` ) {
	chomp $chan;
	next if ( ! -d "out/$chan" );
	if ( `find out/${chan} -name ${chan}.?????????????? | wc -l` > 0 ) {
	    $cmd = "/bin/rm -f out/${chan}/${chan}.?????????????? > /dev/null";
	    return 1 if (&sys_cmd($cmd));
	}
    }

    $rc = &do_tee2ms($TEE2MS_PROD, $opt_m, "out", $yyyy, $ddd);
    return 1 if ($rc);

    return 0;
}


sub parse_time_value {
#
# Parse "ms2csv" output line and return <time,value> pair,
# where time is nominal epoch secs and microsec fraction
# e.g., 2009,1,25,23,59,55.002500,-4032.247333 would return 1232927995.002500,-4032.247333
#
    my ($line) = @_;
    my @a = split /,/, $line;
    my $t = sprintf("%.4d/%.2d/%.2d,%.2d:%.2d:%s",$a[0],$a[1],$a[2],$a[3],$a[4],$a[5]);
    my $time = `caldate -f %N $t`; chomp $time;
    my $value = "$a[6]";
    
    return("$time,$value")
}


sub get_csv {
#
# read CSV file and return <time,value> for first 4 samples
#
    my ($file) = @_;
    my @a = ();
    my $n = 0;

    open(FILE, $file);
    while(<FILE>) {
	next if /^#/;
	chomp;
	push @a, &parse_time_value($_);
	last if ( $n++ >= 3 );
    }
    close(FILE);

    return @a
}


sub is_equal_float {
#
# compare 2 floating point numbers for equality within some tolerance
# if equal return 1 otherwise 0
#
    my ($x, $y, $delta) = @_;

    if ( abs($x - $y) <= $delta ) {
	return 1;
    }
    
    return 0;
}


sub is_match_csv {
#
# compare 1 <time,value> pairs; if match return 1 else 0
#
    my ($a1, $a2) = @_;
    my ($t1, $v1) = split /,/, $a1;
    my ($t2, $v2) = split /,/, $a2;

    if( &is_equal_float( $v1, $v2, $VALUE_TOL ) ) {
	return 1 if( is_equal_float( $t1, $t2, $TIME_TOL ) );
    }
   
    return 0;
}


sub compare_csv {
#
# compare <time, value> for 3 succesive samples at boundary of doys n-1 and n
# allow for 1st sample in each file falling either side of day boundary
#
    my ($file1, $file2) = @_;
    my @a1 = &get_csv($file1); 
    my @a2 = &get_csv($file2);
    
    print "Comparing $file1 and $file2 ...\n" if ($opt_d);

    if( -z "$file1" || -z "$file2" ) {
	return 1;
    }

    if ( &is_match_csv($a1[0], $a2[0]) ) {
	return 0 if ( &is_match_csv($a1[1], $a2[1]) && &is_match_csv($a1[2], $a2[2]));
    }
    
    if ( &is_match_csv($a1[1], $a2[0]) ) {
	return 0 if ( &is_match_csv($a1[2], $a2[1]) && &is_match_csv($a1[3], $a2[2]));
    }

    if ( &is_match_csv($a1[0], $a2[1]) ) {
	return 0 if ( &is_match_csv($a1[1], $a2[2]) && &is_match_csv($a1[2], $a2[3]));
    }

    return 1;
}


sub do_ms2csv {
#
# create and compare CSV file(s) for doys n and n+1 using test channel data
#
    my ($ydoy) = @_;
    my ($yyyy1, $ddd1) = split /\./, $ydoy;
    my ($yyyy2, $ddd2) = split /\./, &next_ydoy($ydoy);
    my $mSEED_file1 = "out/${TEST_CHAN}/${TEST_CHAN}.${yyyy1}.${ddd1}";
    my $mSEED_file2 = "out/${TEST_CHAN}/${TEST_CHAN}.${yyyy2}.${ddd2}";
    my $csv_file1 = "${yyyy1}.${ddd1}.csv";
    my $csv_file2 = "${yyyy2}.${ddd2}.csv";
    my $rc = 0;
    my $cmd;

    # create CSV files for doys n and n+1 using test channel data
    return 1 if ( ! -e $mSEED_file1 || ! -e $mSEED_file2 );

    $cmd = "qmerge -T -f ${yyyy1}.${ddd1},23:59:55 -s 10S $mSEED_file1 2>/dev/null | $MS2CSV > $csv_file1";
    return 1 if(&sys_cmd($cmd));

    $cmd = "qmerge -T -f ${yyyy1}.${ddd1},23:59:55 -s 10S $mSEED_file2 2>/dev/null | $MS2CSV > $csv_file2";
    return 1 if(&sys_cmd($cmd));

    if( ! -z $csv_file1 && ! -z $csv_file2 ) {
	$rc = &compare_csv($csv_file1, $csv_file2);
    }

    if ( ! $rc ) {
	$cmd = "/bin/rm -f $csv_file1 $csv_file2";
	return 1 if(&sys_cmd($cmd));
    }

    return $rc;
}


sub check_overlap {
#
# check mSEED file for overlap, return 1 if failure
#
    my ($file) = @_;
    my $cmd;

    $cmd = "qmerge -n $file 1>/dev/null 2>${file}.tmp";
    return 1 if(&sys_cmd($cmd));

    $cmd = "grep \"Warning: data overlap:\" ${file}.tmp > /dev/null";
    return 1 if(!&sys_cmd($cmd));

    if ( -e "${file}.tmp" ) {
	$cmd = "/bin/rm -f ${file}.tmp";
	return 1 if(&sys_cmd($cmd));
    }

    return 0;
}

sub do_chop {
#
# chop to exact day boundaries, merging with days on either side
#
    my ($ydoy, $ydoy_before, $ydoy_after) = @_;
    my ($yyyy, $ddd) = split /\./, $ydoy;
    
    # chop
    foreach my $chan ( `grep -v "#" $opt_m | awk '{print \$2}'` ) {
	chomp $chan;
	my $file_list = "";
	my $cmd;
	
	next if ( ! -d "out/${chan}" );

	$file_list = "out/$chan/${chan}.${ydoy_before}" if ( -e "out/$chan/${chan}.${ydoy_before}");
	$file_list .= " out/$chan/${chan}.${ydoy}" if ( -e "out/$chan/${chan}.${ydoy}");
	$file_list .= " out/$chan/${chan}.${ydoy_after}" if ( -e "out/$chan/${chan}.${ydoy_after}");

        if( $file_list eq "" ) {
	    warn " Warning: No ${chan} data for ${ydoy}" if ($opt_d);
	    next;
	}

	$cmd = "qmerge $file_list 2>/dev/null | qmerge -T -f ${ydoy},00:00:00 -s 1d > ${chan}.D.${ydoy} 2>/dev/null";
	next if(&sys_cmd($cmd));

	if( ! -e "${chan}.D.${ydoy}" || -z "${chan}.D.${ydoy}" ) {
	    warn " Warning: No trimmed ${chan} data for ${ydoy}" if (!$opt_w);
	    $cmd = "/bin/rm -f ${chan}.D.${ydoy}";
	    &sys_cmd($cmd);
	} else {
	    if ( ! -e "$opt_O/${yyyy}/${ydoy}" ) {
		$cmd = "mkdir -p $opt_O/${yyyy}/${ydoy}";
		return 1 if(&sys_cmd($cmd));
	    }
	    my $dest_file = "$opt_O/${yyyy}/${ydoy}/${chan}.D.${ydoy}";
	    if ( -e "${chan}.D.${ydoy}" ) {
		if ( -e "$dest_file" ) {
		    warn " Warning: $dest_file exists, overwriting ..." if ($opt_d);
		    $cmd = "/bin/rm -f $dest_file";
		    return 1 if(&sys_cmd($cmd));
		}
		$cmd = "/bin/mv -f ${chan}.D.${ydoy} $dest_file";
		return 1 if(&sys_cmd($cmd));
		if( &check_overlap("$dest_file") ) {
		    warn " Warning: Overlap detected for ${ydoy}" if (!$opt_w);
		}
	    }
	}
    }

    return 0;
}


sub do_exceptions {
#
# parse exceptions file and hash missing, discarded and <offset> doys
#
    my ($ydoy, $ydoy_f, $ydoy_t, $diff, $value);
    my $file = $opt_e;
    return 1 if ( ! -e $file );

    open(FILE, $file) or return 1;
    while(<FILE>) {
	next if /^#/; # skip comments
	chomp;
        # print "$_\n";
	if ( /^[\s]*(\d{4}\.\d{3})\-(\d{4}\.\d{3})[\ \t]+(missing|discard|[\s\S]+)/ ) {
	    $ydoy_f = $1;
	    $ydoy_t = $2;
	    $value = $3;
	    # print "$ydoy_f, $ydoy_t, $value\n";
	}
	if ( /^[\s]*(\d{4}\.\d{3})[\ \t]+(missing|discard|[\s\S]+)/ ) {
	    $ydoy_f = $1;
	    $ydoy_t = $1;
	    $value = $2;
	    # print "$ydoy_f, $ydoy_t, $value\n";
	}
	$diff = &diff_ydoy($ydoy_t, $ydoy_f);
	$ydoy = $ydoy_f;
	while($diff >= 0) {
	    # print "$ydoy $value\n";
	    $exceptions{$ydoy} = $value;
	    $ydoy = &next_ydoy($ydoy);
	    $diff = &diff_ydoy($ydoy_t, $ydoy);
	}
    }
    close(FILE);
   
    if ( $opt_d ) {
	# print exceptions hash table
	print "Exceptions table:\n";
	foreach my $key (sort keys %exceptions) {
	    print "$key -> $exceptions{$key}\n";
	}
    }

    return 0;
}

sub do_crank {
#
# turn crank between from and to doys
#
    my ($ydoy_f, $ydoy_t) = @_;
    my $cmd;
    my %done_ydoys = ();
    my $ydoy = $ydoy_f;
    my $diff = &diff_ydoy($ydoy_t, $ydoy_f);
    my $rc = 0;

    # create temporary/unique work dir, copy channel map and change to work dir
    $cmd = "/bin/mkdir -p $WORK_DIR";
    return 1 if(&sys_cmd($cmd));
    $cmd = "/bin/cp -f chan.map $WORK_DIR";
    return 1 if(&sys_cmd($cmd));
    return 1 if ( ! -e "$WORK_DIR" );
    chdir($WORK_DIR);

    # create temporary/unique work dir, copy channel map and change to work dir
    # crank away
    while($diff >= 0) {

	print "Processing $ydoy ... \n";

        # deal with exceptions
	my $skip = 0;
	if ( defined $exceptions{$ydoy} ) {
	    if ( $exceptions{$ydoy} eq "missing" ) {
		warn " Warning: Data missing for $ydoy, ignoring\n" if (!$opt_w);
		$skip = 1;
	    }
	    if ( $exceptions{$ydoy} eq "discard" ) {
		warn " Warning: Data bad for $ydoy, discarding\n" if (!$opt_w);
		$skip = 1;
	    }
	    if ( $exceptions{$ydoy} !~ /missing/ && $exceptions{$ydoy} !~ /discard/ ) {
		warn " Warning: Using option $exceptions{$ydoy} for $ydoy\n" if (!$opt_w);
	    }
	    if ( $skip ) {
	    	$ydoy = &next_ydoy($ydoy);
		$diff = &diff_ydoy($ydoy_t, $ydoy);
		next;
	    }
	}

	my $ydoy_before = &prev_ydoy($ydoy);
	my $ydoy_after = &next_ydoy($ydoy);

	$rc = 0;
	if (!defined $done_ydoys{$ydoy_before}) {
	    $rc = &do_convert($ydoy_before);
	}
	$done_ydoys{$ydoy_before} = 1 if ( !$rc );

	if (!defined $done_ydoys{$ydoy}) {
	    $rc = &do_convert($ydoy);
	}
	$done_ydoys{$ydoy} = 1 if ( !$rc );

	if ( $done_ydoys{$ydoy_before} && $done_ydoys{$ydoy} ) {
	    $rc = &do_ms2csv($ydoy_before);
	    warn " Warning: Data for ${TEST_CHAN} ${ydoy_before} & ${ydoy} do not align\n" if ( $rc );
	}

	if (!defined $done_ydoys{$ydoy_after}) {
	    $rc = &do_convert($ydoy_after);
	}
	$done_ydoys{$ydoy_after} = 1 if ( !$rc );

	$rc = &do_chop($ydoy, $ydoy_before, $ydoy_after);

	# clean up disk space
	foreach (sort(keys %done_ydoys)) {
	    my $key = $_;
	    if (&diff_ydoy($ydoy, $key) > 0) {
		foreach my $chan ( `grep -v "#" $opt_m | awk '{print \$2}'` ) {
		    chomp $chan;
		    if( -e "out/$chan/${chan}.${key}" ) {
			$cmd = "/bin/rm -rf out/$chan/${chan}.${key}";
			&sys_cmd($cmd);
		    }
		}
		delete $done_ydoys{$key};
	    }
	}

	$ydoy = &next_ydoy($ydoy);
	$diff = &diff_ydoy($opt_t, $ydoy);

        # print "\n" if (!$rc);
    }

    # clean up workspace
    if(! $opt_d ) {
	if( -d "${WORK_DIR}" ) {
	    chdir("${WORK_DIR}/..");
	    $cmd = "/bin/rm -rf ${WORK_DIR}";
	    &sys_cmd($cmd);
	}
    }

    return 0;
}


sub do_crank_all {
#
# crank mSEED for all available data
#
    my $rc = 0;
    my $last_day = `/bin/ls -1 $RAW_DIR1/2013 | tail -1`; chomp $last_day;

    return 1 if ( ! &do_crank("2007.137", "2007.365") );
    return 1 if ( ! &do_crank("2008.001", "2008.366") );
    return 1 if ( ! &do_crank("2009.001", "2009.365") );
    return 1 if ( ! &do_crank("2010.001", "2010.365") );
    return 1 if ( ! &do_crank("2011.001", "2011.365") );
    return 1 if ( ! &do_crank("2012.001", "2012.366") );
    return 1 if ( ! &do_crank("2013.001", "2013.${last_day}") );

    return 0;
}


sub do_stats {
#
# print monthly stats between 'from' and 'to' doys
#
    my ($ydoy_f, $ydoy_t) = @_;
    my ($cmd, $tmp);
    my $ydoy = $ydoy_f;
    my $diff = &diff_ydoy($ydoy_t, $ydoy_f);
    my @count = (); # [month][year][[raw/mSEED];
    my @months = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct',  'Nov', 'Dec');

    # initialize array limits
    my $y_f = `caldate -f %Y $ydoy_f`; chomp $y_f;
    my $y_t = `caldate -f %Y $ydoy_t`; chomp $y_t;

    # initialize monthly count of raw/mSEED data for each year
    for(my $m=0; $m < 12; $m++) {
	for(my $y=0; $y < $y_t - $y_f + 1; $y++) {
	    for(my $t=0; $t < 2; $t++) { # raw=0, mSEED=1
		# print "Initializing count[$m][$y][$t] ...\n";
		$count[$m][$y][$t] = 0;
	    }
	}
    }

    while($diff >= 0) {

	my $yyyy = `caldate -f %Y $ydoy`; chomp $yyyy;
	my $ddd = `caldate -f %j $ydoy`; chomp $ddd;
	my $y = $yyyy - $y_f; # array index of year
	my $m = `caldate -f %m $ydoy`; chomp $m; $m -= 1;

	# count raw data
	$tmp = 0;
	if( -e "$RAW_DIR1/$yyyy/$ddd/sfodi1" ) { # femto
	    $tmp = `du --block-size=1 $RAW_DIR1/$yyyy/$ddd/sfodi1 | awk '{print \$1}'`; chomp $tmp;
	} else {
	    if ( -d "${RAW_DIR2}/${yyyy}_${ddd}" ) { #athene
		$tmp = `du --block-size=1 ${RAW_DIR2}/${yyyy}_${ddd} | awk '{print \$1}'`; chomp $tmp;
	    } else {
		if ( -d "${RAW_DIR3}/${yyyy}/${ddd}" ) { #pfemto
		    $tmp = `du --block-size=1 ${RAW_DIR3}/${yyyy}/${ddd} | awk '{print \$1}'`; chomp $tmp;
		} else { # no raw data for this day
		    $tmp = 0;
		}
	    }
	}
	$count[$m][$y][0] += $tmp;

	# count mSEED data
	$tmp = 0;
	if( -e "$DEST_DIR/${yyyy}/${yyyy}.${ddd}" ) {
	    $tmp = `du --block-size=1 $DEST_DIR/${yyyy}/${yyyy}.${ddd} | awk '{print \$1}'`; chomp $tmp;
	    $count[$m][$y][1] += $tmp;
	}
	
	$ydoy = &next_ydoy($ydoy);
	$diff = &diff_ydoy($ydoy_t, $ydoy);
    }

    # pretty print results
    printf("\t");
    for(my $y=$y_f; $y < $y_t + 1; $y++) { # year heading
	printf("%d\t\t", $y);
    }
    printf("\n");
    for(my $y=$y_f; $y < $y_t + 1; $y++) { # raw mSEED heading
	printf("\traw\tmSEED");
    }
    printf("\n");
    
    for(my $m=0; $m < 12; $m++) {
	print "$months[$m]\t";
	for(my $y=0; $y < ($y_t - $y_f + 1); $y++) {
	    for(my $t=0; $t < 2; $t++) { # raw=0, mSEED=1
		printf("%.2f\t", $count[$m][$y][$t]/1000000000.0);
	    }
	}
	printf("\n");

    }

    printf("*numbers are in GB, where 1GB=1.0e9 bytes\n");

    return;
}

sub reg_test1 {
#
# 1-day comparison
#
    my ($n, $save, $ydoy) = @_;
    my ($yyyy, $ddd) = split /\./, $ydoy;
    my ($cmd, $rc);

    print "test#${n} converting ${yyyy}.${ddd} to mSEED ... ";

    # clean test out dir before starting
    $cmd = "/bin/rm -rf out/*";
    return 1 if(&sys_cmd($cmd));

    $cmd = "/bin/cp -f ../input/${TEST_CHAN}.map .";
    return 1 if(&sys_cmd($cmd));

    $rc = &do_tee2ms($TEE2MS_TEST, "${TEST_CHAN}.map", "out", $yyyy, $ddd);
    return 1 if ($rc);

    $cmd = "qmerge -n -i d0 out/${TEST_CHAN}/\* > ${yyyy}.${ddd}_qmerge.log 2>&1";
    return 1 if(&sys_cmd($cmd));

    # make master copies of test output
    if ($save) {
	$cmd = "/bin/cp -f ${yyyy}.${ddd}_qmerge.log ../output";
	return 1 if(&sys_cmd($cmd));
    } else {
	$cmd = "diff ../output/${yyyy}.${ddd}_qmerge.log ${yyyy}.${ddd}_qmerge.log";
	return 1 if(&sys_cmd($cmd));
    }

    return 0;
}


sub reg_test2 {
#
# 2-day overlap comparison
#
    my ($n, $save, $ydoy1, $ydoy2) = @_;
    my ($yyyy1, $ddd1) = split /\./, $ydoy1;
    my ($yyyy2, $ddd2) = split /\./, $ydoy2;
    my ($cmd, $rc);

    print "test#${n} aligning ${yyyy1}.${ddd1} & ${yyyy2}.${ddd2} ... ";

    # clean test out dir before starting
    $cmd = "/bin/rm -rf out/*";
    return 1 if(&sys_cmd($cmd));
   
    $cmd = "/bin/cp -f ../input/${TEST_CHAN}.map .";
    return 1 if(&sys_cmd($cmd));

    $rc = &do_tee2ms( $TEE2MS_TEST, "${TEST_CHAN}.map","out",  $yyyy1, $ddd1 );
    return 1 if ($rc); 

    $rc = &do_tee2ms( $TEE2MS_TEST, "${TEST_CHAN}.map", "out", $yyyy2, $ddd2 );
    return 1 if ($rc); 

    $rc = &do_ms2csv( "${yyyy1}.${ddd1}" );
    return 1 if(&sys_cmd($cmd));
   
    return 0;
}


sub do_test {
#
# run regression testing on a small amount of data
#
    my $n = 1; # test number
    my $cmd;

    return 1 if ( ! -e "$TEST_DIR" );
    chdir("$TEST_DIR");

    # clean test work dir before starting
    $cmd = "/bin/rm -rf work/*";
    return 1 if(&sys_cmd($cmd));

    chdir("work");

    # compare mSEED output for problem days
    foreach my $ydoy ( "2008.015", "2008.238", "2008.366", "2009.001", "2011.254", "2011.316", "2011.323", "2012.010", "2012.021", "2012.182", "2012.183", "2013.003" ) {
	if ( &reg_test1( $n++, 0, $ydoy ) ) {
	    print "failed\n";
	} else {
	    print "ok\n";
	}
    }

    # compare consecutive days for overlap match to within 1 sample
    foreach my $ydoy ( "2008.366", "2009.001", "2009.002", "2009.025", "2009.156", "2009.352", "2011.316", "2011.325", "2012.010", "2012.182", "2012.183" ) {
	if ( &reg_test2( $n++, 0, $ydoy, &next_ydoy($ydoy) ) ) {
	    print "failed\n";
	} else {
	    print "ok\n";
	}
    }

    # clean test work dir after finishing
    chdir("$TEST_DIR");
    $cmd = "/bin/rm -rf work/*";
    return 1 if(&sys_cmd($cmd));
  
    return 0;
}


{
    STDERR->autoflush(1);
    STDOUT->autoflush(1);

    $opt_m = "chan.map";
    $opt_e = "$EXCEPTIONS";
    $opt_d = 0;
    $opt_w = 0;
    $opt_r = 0;
    $opt_a = 0;
    $opt_s = 0;
    $opt_O = "$DEST_DIR";

    &getopts ('hf:t:dwro:e:aO:s');
    &help() if ($opt_h);

    $opt_w = 0 if ($opt_d);

    # create exceptions hash table
    &do_exceptions();

    # set leap secs file for qlib2
    my $os = `uname -s`; chomp $os;
    $ENV{LEAPSECONDS} = "${QLIB2_DIR}/leapseconds";
    warn " Warning: Leapsecs file set to $ENV{LEAPSECONDS}\n" if (!$opt_w);

    if ( $opt_a ) {
	&do_crank_all();
    }

    if ( $opt_r ) {
	&do_test();
    }

    if ( $opt_f && $opt_t ) {
	if ($opt_s) {
	    &do_stats( $opt_f, $opt_t );
	    exit(0);
	}
	&do_crank( $opt_f, $opt_t );
    }

    exit(0);
}
