/*======================================================================
 *
 * Program to demultiplex and convert IDA 10 format data into mSEED
 * Author: Mario Aranha, Berkeley Seismological Laboratory
 * Copyright (C) 2013 Regents of the University of California.
 *
 *====================================================================*/
#include <stdio.h>
#include <unistd.h>
#include <search.h>
#include <string.h>
#include <errno.h>
#include <memory.h>
#include <stdlib.h>
#include <values.h>

#include "qlib2.h"
#include "tee2ms.h"

#define VERSION "1.1.0 (2013.183)"
#define	FIXED_DATA_HDR_SIZE 48

/* external variables */
extern char *optarg;
extern int  optind;

/* local variables */
static int debug = 0;
static int warn = 1;
static char outdir[MAXSTR];
static char mapfile[MAXSTR];
static int output_datatype = UNKNOWN_DATATYPE;
static int mseed_blksize = DEFAULT_BLKSIZE;
static int num_sncl = 0;
static int date_fmt = JULIAN_FMT;
static int no_gzip = 0;
static int statistics = 0;
static int timing_opt = 0;
static double errtol = 0.00025; /* seconds */
static UINT64 init_offset = 0; /* 'ext - sys' value in tics */
static UINT64 start_new = 0; /* 'sys' value in tics */
static UINT64 discard_from = 0; /* 'sys' value in tics */
static struct hash_info h[MAX_SNCL];


/************************************************************************/
/*	Leap second time structures. (from "qtime.c")		        */
/************************************************************************/
/************************************************************************/
/*	Information on dates (for non-leap years)			*/
/*		Names of months, days per months, and day of year.	*/

static char	    *MON[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", 
			"Jul", "Aug", "Sep", "Oct", "Nov", "Dec", NULL};
/*		         Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec */
static int	    DPM[] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
static int	    DOY [] = { 0, 31, 59, 90,120,151,181,212,243,273,304,334,365 };

/*  Define max number of leaps we can handle if system does not define.	*/
/*  Use this definition if you supply your own leapsecond file.		*/
#ifndef	    LEAPSECONDS
#define	    LEAPSECONDS	    "/usr/local/lib/leapseconds"
#endif

/*  Use this definition if your system comes with a leapsecond file.	*/
#ifndef	    LEAPSECONDS
#define	    LEAPSECONDS	    "/usr/share/lib/zoneinfo/leapseconds"
#endif

#ifndef	    TZ_MAX_LEAPS
#define	    TZ_MAX_LEAPS    100
#endif

#define LEAPLINELEN	255


typedef struct lsinfo {
    EXT_TIME	exttime;	/*  External def. of leap time.		*/
    INT_TIME	inttime;	/*  Internal def. of leap time,		*/
				/*  incl. prior leaps this year.	*/
    int		leap_value;	/*  Leap increment in seconds.		*/
    int		total_offset;	/*  New total epoch time offset: (t-n)	*/
    double	tepoch;		/*  True epoch time of leap time.	*/
    double	nepoch;		/*  Nominal epoch time of leap time.	*/
} LSINFO;

struct lstable {
    int		initialized;	/*  Leap second table inited?		*/
    int		nleapseconds;	/*  Total leap second entries.		*/
    LSINFO	lsinfo[TZ_MAX_LEAPS];
				/*  Info for each leapsecond.		*/
} lstable_local;


/************************************************************************/
/*  init_leap_second_table:						*/
/*	Initialize leap second table from external file.		*/
/*  Return: 0 on success, 1 on warning, -1 on error.			*/
/************************************************************************/
int init_leap_second_table_local()
{
    FILE    *lf;
    char    line[LEAPLINELEN+1], keywd[10], s_month[10], corr[10], type[10];
    int	    i, l, ls, n, delta_second;
    int	    lnum = 0;
    char    leap_file[MAXPATHLEN];
    char    *ep;
    LSINFO  *p;
    EXT_TIME et;
    struct stat stat_buf;
    static int status = 0;
    INT_TIME prev_inttime;
    INT_TIME inttime_epoch_origin = {1970,0,0};
    double sdif;

    if (lstable_local.initialized) return (status);
    lstable_local.initialized = 1;

#ifdef	NO_LEAPSECONDS
    /* If people want to use qlib2 without leapsecond inclusion,	*/
    /* they can define NO_LEAPSECONDS in the Makefile.			*/
    if (1) return (status);
#endif

    /*	If the environment variable LEAPSECONDS exists, it should	*/
    /*	override the default LEAPSECONDS file.				*/
    if ((ep=getenv("LEAPSECONDS"))!=NULL)
	strcpy(leap_file,ep);
    else strcpy(leap_file, LEAPSECONDS);

    /* Check to make leapsecond is either a file or char device.	*/
    if (stat(leap_file, &stat_buf) == 0 && 
	! (S_ISREG(stat_buf.st_mode) || S_ISCHR(stat_buf.st_mode))) {
	fprintf (stderr, "Error: invalid leapsecond file %s\n", leap_file);
	return (-1);
    }

    if ((lf=fopen(leap_file, "r"))==NULL) {
	fprintf (stderr, "Warning: unable to open leap second file: %s\n",leap_file);
	return (1);
    }

    while (status == 0 && fgets(line,LEAPLINELEN,lf)!=NULL) {
	++lnum;
	line[LEAPLINELEN]='\0';
	trim(line);
	l = strlen(line);
	if (l>0 && line[l-1]=='\n') line[--l] = '\0';
	if (l<=0 || line[0]=='#') continue;
	if ((ls=lstable_local.nleapseconds) >= TZ_MAX_LEAPS) {
	    fprintf (stderr, "Error: too many leapsecond entries - line %d\n", lnum);
	    fflush (stderr);
	    if (QLIB2_CLASSIC) exit(1);
	    status = QLIB2_TIME_ERROR;
	    continue;
	}
 	p = &lstable_local.lsinfo[ls];
	if ((l=sscanf(line,"%s %d %s %d %d:%d:%d %s %s", keywd, &p->exttime.year, 
		      s_month, &p->exttime.day, &p->exttime.hour, &p->exttime.minute, 
		      &p->exttime.second, corr, type))!= 9) {
	    fprintf (stderr, "Error: invalid leapsecond line - line %d\n", lnum);
	    fflush (stderr);
	    if (QLIB2_CLASSIC) exit(1);
	    status = QLIB2_TIME_ERROR;
	    continue;
	}
	i = 0;
	while (MON[i] != 0) {
	    if (strcasecmp(MON[i],s_month)==0) {
		p->exttime.month = i+1;
		p->exttime.doy = DOY[i] + p->exttime.day +
		    ( IS_LEAP(p->exttime.year) && (p->exttime.month > 2) );
		break;
	    }
	    ++i;
	}
	p->exttime.usec = 0;
	switch (corr[0]) {
	    case '+':	p->leap_value = 1; break;
	    case '-':	p->leap_value = -1; break;
	    default:
		p->leap_value = 0;
		fprintf(stderr, "Error: invalid leapsecond correction - line %d\n", lnum);
		fflush (stderr);
		if (QLIB2_CLASSIC) exit(1);
		status = QLIB2_TIME_ERROR;
		break;
	}
	if (p->leap_value == 0) continue;

	/* Make sure we can handle consecutive leapseconds. */
	/* Both leapseconds will map to the same nominal time. */
	n = lstable_local.nleapseconds;
	et = p->exttime;
	if (p->leap_value == 1) {
	    /* Positive leap second: */
	    /* Compute nominal and true epoch time of last second with 1-1 mapping  */
	    /* before this leap second. */
	    et = p->exttime;
	    et.second = 59;
	    delta_second = p->exttime.second - et.second;
	    p->inttime = ext_to_int (et);
	    p->tepoch = int_to_tepoch (p->inttime);
	    p->nepoch = int_to_nepoch (p->inttime);
	    /* Now define the values for this leapsecond. */
	    /* Compute nominal and true epoch times for this second. */
	    /* Nominal time for a POSITIVE leapsecond is defined as the */
	    /* nominal time of the next nominal second. */
	    p->inttime.second += delta_second;
	    p->tepoch += delta_second;
	    p->nepoch += 1;
	    p->total_offset = (n > 0) ? lstable_local.lsinfo[n-1].total_offset + p->leap_value : p->leap_value;
	}
	else {	
	    /* Negative leap second: */
	    /* Compute nominal and true epoch time of (worst case) last second with 1-1 mapping  */
	    /* before this leap second. */
	    et = p->exttime;
	    et.second = 57; /* There can never be more than 2 consecutive negative leap seconds. */
	    delta_second = p->exttime.second - et.second;
	    p->inttime = ext_to_int (et);
	    p->tepoch = int_to_tepoch (p->inttime);
	    p->nepoch = int_to_nepoch (p->inttime);
	    /* Now define the values for this leapsecond. */
	    /* Compute nominal and true epoch times for this second. */
	    /* Nominal time for a NEGATIVE leapsecond is defined as the */
	    /* nominal time of the next nominal second. */
	    p->tepoch += delta_second;
	    p->nepoch += delta_second;
	    p->inttime.second += delta_second; /* IS THIS CORRECT ? */
	    p->total_offset = (n > 0) ? lstable_local.lsinfo[n-1].total_offset + p->leap_value : p->leap_value;
	}
	/* Sanity check - ensure entries are in increasing time order. */
	prev_inttime = (n > 0) ? lstable_local.lsinfo[n-1].inttime : inttime_epoch_origin;
	sdif = tdiff (p->inttime, prev_inttime) / USECS_PER_SEC;
	if (sdif < 0) {
	    fprintf (stderr, "Error: leapsecond table not in increasing time order.\n");
	    fprintf (stderr, "%s\n", line);
	    fflush (stderr);
	    if (QLIB2_CLASSIC) exit(1);
	    status = QLIB2_TIME_ERROR;
	    continue;
	}
	++lstable_local.nleapseconds;
    }
    if (ferror(lf)) {
	fprintf (stderr, "Error: reading leap second file: %s\n",leap_file);
	status = -1;
    }
	
    fclose(lf);
    return (status);
}

/************************************************************************/
/*  is_leap_second_n: (from "qtime.c")							*/
/*	Return lsinfo structure if nominal second is in leap second table*/
/************************************************************************/
static LSINFO *is_leap_second_n
   (double nepoch)		/* Nominal epoch time			*/
{
    int		i;
    int		status;
    /*	Search leap second table.   */
    if (!lstable_local.initialized) {
	status = init_leap_second_table_local();
    }
    nepoch = floor(nepoch);
    for (i=lstable_local.nleapseconds-1; i>=0; i--) {
	if (nepoch == lstable_local.lsinfo[i].nepoch)
	    return (&lstable_local.lsinfo[i]);
    }
    return(NULL);
}

/* command line usage */
static char *syntax[] = {
"%s version " VERSION " -- Convert multi-channel IDA10 data into single channel mSEED",
"%s [-h] [-o outdir] [-d] [-w] [-b blksize] [-m mapfile] [-z] [-O offset] [-s] [-T option] [-e errtol] [-S sys] [-D sys] [input_file(s)]",
"    where:",
"	-h	    Help - prints syntax message.",
"	-o outdir   Output directory for files. Default is dmxdata.",
"	-d 	    Turn on debug messages",
"	-w 	    Turn off warning messages",
"	-b blksize  Specify output blksize in bytes.",
"		    Must be between 256 and 8192 inclusive, and blksize = 2^N.",
"		    Default blksize is 4096, max is 8192.",
"	-m mapfile  Specifies station and channel name mappings. Default is chan.map.",
"	-z	    Input files not gzip'd.",
"	-O offset   Apply <offset> (tics) initially when clock status is bad.",
"	-s 	    Print statistics",
"	-T option   Timing option.",
"                       0 = buffer data and compute average sample rate over ~daily file using blockette 100 (default)",
"                       1 = new timeseries when (cumulative) error > threshold, no b100",
"                           when error > 0.5*sint => reset seq=1, error > errtol => increment seq_no",
"	-e errtol   Error tolerance in secs (default=0.00025)",
"	-S sys      Force start of new timeseries at sys=<sys> (tics)",
"	-D sys      Discard packets from sys=<sys> (tics)",
"       input_file  IDA10 data file (default is gzip'd). Can also input uncompressed data or files(s) from stdin",
NULL };


static int tee2ms_get_blksize(char *str)
     /* parse blksize_str and return blocksize;
	must be power of 2 and within limits */
{
    int blksize, n, m;
    char *p;

    blksize = strtol(str, &p, 10);
    switch(*p) {
      case 0:	break;
      case 'k':
      case 'K':	blksize *= 1024; break;
      default:	blksize = 0;
    }

    if (blksize <= 0 || blksize > MAX_BLKSIZE) {
      return 0;
    }

    n = (int)log2((double)blksize);
    m = (int)pow(2.0,(double)n);
    if (m != blksize) {
      return 0;
    }

    return blksize;
}


static void tee2ms_cmdlineargs(int argc, char **argv)
     /* read command line arguments */
{

    int c;
    char *cmdname = argv[0];

    /* set these names to defaults */
    (void)strcpy(outdir, ".");
    (void)strcpy(mapfile, "chan.map");
   
    while ( (c = getopt(argc, argv, "ho:dwb:m:zO:sT:e:S:D:")) != -1) {
      switch (c) {
      case '?':
      case 'h':
	print_syntax(cmdname, syntax, stderr);
	exit(0);
      case 'o':
	(void)strcpy(outdir, optarg);
	break;
      case 'd':
	debug = 1;
	break;
      case 'w':
	warn = 0;;
	break;
      case 'b':
	mseed_blksize = tee2ms_get_blksize(optarg);
	if(mseed_blksize == 0) {
	   mseed_blksize = DEFAULT_BLKSIZE;
           if(warn) {
	     (void)fprintf(stderr, "Warning: Unsupported blocksize, setting to default %d\n", mseed_blksize); 
           }
	}
	break;
      case 'm':
	(void)strcpy(mapfile, optarg);
	break;
      case 'z':
	no_gzip = 1;
	break;
      case 'O':
	init_offset = atoll(optarg);
	(void)fprintf(stderr, "init_offset=%lld(tics)\n", init_offset);
	break;
      case 's':
	statistics = 1;
	break;
      case 'T':
	timing_opt = atoi(optarg);
	break;
      case 'e':
	errtol = atof(optarg);
	break;
      case 'S':
	start_new = atoll(optarg);
	(void)fprintf(stderr, "force new timeseries at sys=%lld(tics)\n", start_new);
	break;
     case 'D':
	discard_from = atoll(optarg);
	(void)fprintf(stderr, "discard packets start at sys=%lld(tics)\n",discard_from);
	break;
      default:
	(void)fprintf (stderr, "Unknown option: -%c\n", c);
	exit(1);
      }
    }

    return;
}


static int tee2ms_output_datatype(int input_type)
     /* mSEED output data type */
{
  int output_type;

  switch(input_type) {
  case IDA10_DATA_INT32:
  case IDA10_DATA_INT16:
  case IDA10_DATA_INT8:
    output_type = STEIM2;
    break;

  case IDA10_DATA_REAL64:
    output_type = IEEE_FP_DP;
    break;

  case IDA10_DATA_REAL32:
    output_type = IEEE_FP_SP;
    break;

  case IDA10_DATA_UNKNOWN:
  default:
      /* unknown */
      if(warn) {
	(void)fprintf(stderr, "Warning: Input datatype %d unknown\n", input_type);
      }
      output_type = UNKNOWN_DATATYPE;
      break;
  }

  return output_type;
}


static DATA_HDR* tee2ms_create_hdr(IDA10_TSHDR *ihdr, int i)
     /* creat SEED header */
{
    char s[MAXSTR], n[MAXSTR], c[MAXSTR], l[MAXSTR], tmpstr[MAXSTR];

    /* always use BIG_ENDIAN */
    set_hdr_wordorder(SEED_BIG_ENDIAN);
    set_data_wordorder(SEED_BIG_ENDIAN);
    set_record_type('Q');

    DATA_HDR *shdr = new_data_hdr();
    if(debug) {
      (void)fprintf (stderr, "Debug: Creating new mSEED header, sncl=%s\n", h[i].sncl);
    }

    if( !shdr ) {
      return NULL;
    } 

#ifdef FOO
    /* always use BIG_ENDIAN - see above default settings */
    shdr->hdr_wordorder = SEED_BIG_ENDIAN;
    shdr->data_wordorder = SEED_BIG_ENDIAN;
#endif

    (void)sscanf(h[i].sncl, "%s.%s.%s.%s", s, n, c, l);
    (void)strcpy (tmpstr, h[i].sncl);
    (void)strcpy (shdr->station_id, strtok(tmpstr, "."));
    (void)strcpy (shdr->network_id, strtok(NULL, "."));
    (void)strcpy (shdr->channel_id, strtok(NULL, "."));
    (void)strcpy (shdr->location_id, strtok(NULL, "."));

    shdr->sample_rate = roundoff(ihdr->srate); /* assumes rate >= 1 sps */
    shdr->sample_rate_mult = 1; /* same as ihdr->srate.multiplier ? */

    shdr->data_type = tee2ms_output_datatype(ihdr->datatype);
    shdr->blksize = mseed_blksize;
    shdr->pblockettes = NULL;
    
    shdr->hdrtime = nepoch_to_int(ihdr->cmn.beg); /* this is nominal 1970 epoch time */
    shdr->begtime = nepoch_to_int(ihdr->cmn.beg);
    shdr->seq_no = 1;
    shdr->xm1 = 0;

    return shdr;
}


#ifdef FOO
static int tee2ms_add_blockette_1001(DATA_HDR *shdr, int i)
  /* add blockette 1001 to mSEED header for microsecond time precision */
{
  BLOCKETTE_1001 b;
  int ok;

  b.hdr.type = 1001;
  b.hdr.next = 0;
  b.clock_quality = 100;
  b.flags = 0;
  b.frame_count = 0; /* updated in ms_pack2_* functions which call init_sdr_hdr() */
  b.usec99 = 0;      /* updated in ms_pack2_* functions which call init_sdr_hdr() */

  if (my_wordorder < 0) get_my_wordorder();

  ok = add_blockette(shdr, (char*)&b, 1001, sizeof(BLOCKETTE_1001), my_wordorder, -1);
 
  return ok;
}
#endif


static int tee2ms_add_blockette_100(DATA_HDR *shdr, int i)
  /* add blockette 100 to mSEED header */
{
  BLOCKETTE_100 b;
  int ok;

  b.hdr.type = 100;
  b.hdr.next = 0;
  b.actual_rate = h[i].av_rate;
  b.flags = 0;

  if (my_wordorder < 0) get_my_wordorder();
  ok = add_blockette(shdr, (char*)&b, 100, sizeof(BLOCKETTE_100), my_wordorder, -1);
 
  return ok;
}


static void tee2ms_reset_timeseries(int i, IDA10_TSHDR *ihdr, BOOL clock_ok, BOOL reset_seq_no)
     /* start new timeseries */
{
    if(!h[i].shdr) {
      return;
    }

    /* save first packet of timeseries */
    h[i].first_ihdr = *ihdr;

    /* reset mSEED header info */
    h[i].shdr->hdrtime = nepoch_to_int(h[i].first_ihdr.cmn.beg);
    h[i].shdr->begtime = h[i].shdr->hdrtime;
    if( reset_seq_no ) {
      /* reset for larger error, increment for small time errors */
      h[i].shdr->seq_no = 1;
      h[i].shdr->xm1 = 0;
    }

    /* reset other info */
    h[i].nout = 0;
    h[i].ts_nsamp = 0;
    h[i].ts_npkt = 0;
    h[i].av_rate = 0.0;
    if( init_offset != 0 && (h[i].is_first_pkt || (!clock_ok && !reset_seq_no)) ) {
      h[i].first_offset = init_offset * NANOSEC_PER_OFISTIC;
      h[i].is_questionable_ttag = TRUE;
    } else {
      h[i].first_offset = 0;
      h[i].is_questionable_ttag = FALSE;
    }
    h[i].is_reboot = FALSE;
    if( reset_seq_no ) {
      /* significant timing error */
      h[i].curr_offset = 0;
    }

    /* reset file statistics */
    if( h[i].is_first_pkt ) {
      h[i].error.pkt = h[i].first_ihdr;
      h[i].error.value = -MAXDOUBLE;
      h[i].is_sequential = TRUE;
    }
    h[i].error.ts_nsamp = 0;
 
    if(debug) {
      (void)fprintf(stderr, "Debug: %s *new timeseries* start seq=%d first_offset=%lld(tics) curr_offset=%lld(tics)\n",
		  h[i].sncl, h[i].shdr->seq_no, h[i].first_offset/NANOSEC_PER_OFISTIC, h[i].curr_offset/NANOSEC_PER_OFISTIC);
    }

    return;
}


static void tee2ms_hash_reset(int i)
     /* initialize/reset hash info for current file */
{
  h[i].nalloc = 0;
  h[i].nout = 0;
  h[i].outf = (FILE*)NULL;
  h[i].shdr = NULL;
  h[i].databuf.p_int = NULL;
  h[i].databuf.p_float = NULL;
  h[i].databuf.p_double = NULL;
  h[i].ts_nsamp = 0;
  h[i].ts_npkt = 0;
  h[i].chan_nsamp = 0;
  h[i].chan_npkt = 0;  
  h[i].first_offset = 0;
  h[i].is_first_pkt = TRUE;
  h[i].first_valid_offset = 0;
  h[i].last_valid_offset = 0;
  h[i].large_offset = FALSE;

  return;
}


static int tee2ms_hash_new(char *channel, char *sncl)
     /* create new hash table entry */
{
  ENTRY sentry, *hentry;
  int i;

  /* hash key is original channel name */
  sentry.key = channel;
  if ((hentry=hsearch(sentry, FIND)) != NULL) {
    i = ((struct hash_info *)hentry->data)->index;
    return i;
  }

  i = num_sncl++;

  sentry.key = strdup(channel);
  sentry.data = (void*)(h+i);
  if ((hentry=hsearch(sentry, ENTER)) == NULL) {
    (void)fprintf (stderr, "Error: entering hash entry for %s\n", sentry.key);
    exit(1);
  }

  /* initialize hash info for this channel */
  h[i].key = sentry.key;
  (void)strcpy(h[i].sncl, sncl);
  h[i].index = i;

  tee2ms_hash_reset(i);

  if(debug) {
    (void)fprintf(stderr, "Debug: tee2ms_hash_new: created hash entry i=%d channel=%s sncl=%s\n", i, channel, sncl); 
  }

  return i;
}


static int tee2ms_open_output_file(int i) {
  mode_t mode = 0644, mode_dir = 0755;
  char dirname[MAXSTR], outfile[MAXSTR];
  EXT_TIME et = int_to_ext(h[i].shdr->hdrtime);

  if(h[i].outf) {
    return 0;
  }

  /* quietly use dir if it exists */
  if(mkdir(outdir, mode_dir) != 0 && errno != EEXIST) {
      (void)fprintf (stderr, "Error: Cannot create directory %s\n", outdir);
      return 1;
  } else {
      if (debug) {
	 (void)fprintf(stderr, "debug: Created output dir %s\n", outdir);
      }
      errno = 0;
  }

  (void)snprintf (dirname, sizeof(dirname), "%s%c%s", outdir, PATH_DELIMITER,
		  h[i].sncl);

  (void)snprintf (outfile, sizeof(outfile), "%s%c%s.%04d%02d%02d%02d%02d%02d",
		  dirname, PATH_DELIMITER, h[i].sncl,
		  et.year, et.month, et.day, et.hour, et.minute, et.second);

  if(mkdir(dirname, mode_dir) != 0 && errno != EEXIST) {
      (void)fprintf (stderr, "Error: Cannot create directory %s\n", dirname);
      return 1;
  } else {
      if (debug) {
	 (void)fprintf(stderr, "Debug: Created output sub-dir %s\n", dirname);
      }
      errno = 0;
  }

  if ((h[i].outf = fopen (outfile, "wb")) == NULL) {
      (void)fprintf (stderr, "Error: Opening file %s file#=%d errno=%d\n",
			   outfile, i, errno);
      perror(outfile);
      return 1;
  }

  return 0;
}


static int tee2ms_close_output_file(int i) {
  int ret_val;

  if( !h[i].outf ) {
    return 0;
  }

  ret_val = fclose(h[i].outf);

  if( ret_val == 0) {
    h[i].outf = (FILE*)NULL;
  }

  return ret_val;
}


static int tee2ms_hash_update(IDA10_TSHDR *ihdr)
  /* initialize channel info and return hash index for data channel, -1 if error */
{
  int i;
  size_t nalloc = 0;
  ENTRY sentry, *hentry;
  char *key = ihdr->cname;
  IDA10_TTAG ttag;

  /* Hash key is channel name */
  sentry.key = key;
  if ((hentry = hsearch(sentry, FIND)) == NULL) {  /* not found */
#ifdef DEBUG
    if(warn) {
      (void)fprintf(stderr, "Warning: No mapping for channel %s, ignoring\n", key);
    }
#endif
    return -1;
  } else { /* found hash entry */
    i = ((struct hash_info *)hentry->data)->index;

    if( i < 0 || i > MAX_SNCL ) {
      return i;
    }   
    
    /* create and initialize SEED header */
    if(h[i].shdr == NULL) {
      h[i].shdr = tee2ms_create_hdr(ihdr, i);
      if( h[i].shdr == NULL) {
	(void)fprintf (stderr, "Error: unable to create mSEED header\n");
	exit(1);
      }
    }
  }

  return i;
}


static void tee2ms_read_mapfile()
  /* read map file and create hash entry for each channel */
{
  FILE *fp;
  char line[MAXLINE], s1[MAXSTR], s2[MAXSTR], s3[MAXSTR];
  int i;

  if ((fp = fopen (mapfile, "r")) == NULL) {
      (void)fprintf (stderr, "Error: Opening map file %s ferrno=%d\n",
		     mapfile, errno);
      perror(mapfile);
      exit(1);
  }

  while(fgets(line, MAXLINE, fp)) {
    if(line[0] == '#') {
      continue;
    }

    sscanf(line, "%s %s %s\n", s1, s2, s3);
#ifdef DEBUG
    if(debug) {
      (void)fprintf(stderr, "%s %s %s\n", s1, s2, s3);
      /* fputs(line, stderr); */
    }
#endif

    tee2ms_hash_new(s1, s2);
  }

  (void)fclose(fp);

  return;
}


static void tee2ms_init()
     /* initialize various things */
{
  /* Initialize hash table */
  if (hcreate (MAX_HASH) == 0) {
    (void)fprintf (stderr, "tee2ms_init: Error: Unable to create hash table\n");
    exit(1);
  }

  /* read channel map file */
  tee2ms_read_mapfile();
    
  /* initialize to QLIB2_NOEXIT mode */
  init_qlib2(1);

  return;
}

static void tee2ms_remove_blanks(char *string)
     /* remove blank chars from string */
{
    int i;

    for (i = 0; i < (int) strlen(string); i++) if (string[i] == ' ') string[i] = '_';

    return;
}


static int tee2ms_pkt_read(FILE *fp, IDA10_TS *ts)
     /* read IDA10 record */
{
    int status, RecordType;
    static UINT8 buf[IDA10_MAXRECLEN];

    while (1) {
        status = ida10ReadRecord(fp, buf, IDA10_MAXRECLEN, &RecordType, FALSE);
        if (status < 0) {
            if (status == IDA10_EOF) return FALSE;
            (void)fprintf(stderr, "Error: tee2ms_read_pk: ida10ReadRecord error "),
            (void)fprintf(stderr, "%d at record ", status);
            (void)fprintf(stderr, "%s\n", ida10ErrorString(status));
            return TRUE;
        }

	/* skip unrecognized packet types */
        if ( RecordType != IDA10_TYPE_TS ) {
	  if (warn) {
            (void)fprintf(stderr, "Warning: IDA10 input packet of type %d ignored\n", RecordType);
	  }
          continue;
        }

        if (ida10UnpackTS(buf, ts)) {
	  tee2ms_remove_blanks(ts->hdr.sname);
          tee2ms_remove_blanks(ts->hdr.cname);
          return TRUE;
        } else {
	  if(warn) {
            (void)fprintf(stderr, "Warning: Can't decode IDA10 input packet\n");
	  }
	  return FALSE;
        }
    }
}


static void tee2ms_get_ttag_info(IDA10_TSHDR *ihdr, UINT64 *ext, UINT64 *sys, UINT64 *pps, UINT64 *offset)
    /* return OFIS time tag parameters from IDA10 header; all returned values in nonosecs */
{
    IDA10_OFISTAG ttag = ihdr->cmn.ttag.beg.ofis; /* OFIS time tag */

    *ext = (UINT64)ttag.ext * NANOSEC_PER_SEC; /* GPS(external) time in nanosecs since SAN epoch 1/1/1999 */
    *sys = ttag.sys * NANOSEC_PER_OFISTIC;  /* Femtometer system time in nanosec */
    *pps = ttag.pps * NANOSEC_PER_OFISTIC;  /* PPS(1-Hz) time in nanosec */
    *offset = (*ext) - (*pps); /* nanosecs */

    return;
}


static void tee2ms_print_ttag_info(IDA10_TSHDR *ihdr)
    /* print OFIS time tag parameters from IDA10 header */
{
    char time_str_beg[MAXSTR], time_str_end[MAXSTR];
    /* IDA10 'end' time is at start of last sample in packet */
    (void)sprintf(time_str_beg, "%s", time_to_str(nepoch_to_int(ihdr->cmn.beg), date_fmt));
    (void)sprintf(time_str_end, "%s", time_to_str(nepoch_to_int(ihdr->cmn.end + ihdr->sint), date_fmt));
    (void)fprintf(stderr, " rate=%.4f(sps) nsamp=%d sint=%lld(tics) incerr=%lld(tics)",
		  ihdr->srate, ihdr->nsamp, ihdr->nsint/NANOSEC_PER_OFISTIC, ihdr->incerr.nsec/NANOSEC_PER_OFISTIC);
    (void)fprintf(stderr, " beg=%s", time_str_beg);
    (void)fprintf(stderr, " end*=%s", time_str_end);
    (void)fprintf(stderr, "\n");

    IDA10_OFISTAG ttag = ihdr->cmn.ttag.beg.ofis; /*  OFIS 'beg' time tag */
    UINT64 offset = ihdr->cmn.ttag.beg.offset/NANOSEC_PER_OFISTIC;
    (void)fprintf(stderr, " ext=%lld(s) sys=%lld(tics) pps=%lld(tics) offset=%lld(tics) beg=%f(s) end=%f(s)\n",
		  (UINT64)ttag.ext, ttag.sys, ttag.pps, offset, ihdr->cmn.beg, ihdr->cmn.end );

    (void)fprintf(stderr, " init=%s pps=%s avail=%s suspect=%s locked=%s leapsec=%s\n",
		  ttag.status.init ? "yes" : "no", ttag.status.pps ? "yes" : "no", ttag.status.avail ? "yes" : "no",
		  ttag.status.suspect ? "yes" : "no", ttag.status.locked ? "yes" : "no", ttag.status.leapsec ? "yes" : "no");

    return;
}


static void tee2ms_fix_timestamp(IDA10_TSHDR *ihdr, UINT64 sys, UINT64 offset)
    /* fix timestamps in IDA10 headers, offset is in nanosecs */
{
    REAL64 EpochYearFactor = 0.0;
    REAL64 duration = ihdr->sint * (ihdr->nsamp - 1); /* secs */
    REAL64 epoch;
    IDA10_OFISTAG ttag;

    /* fix beg */
    epoch =  ((REAL64) (sys + offset) / (REAL64)NANOSEC_PER_SEC) + EpochYearFactor;
    ihdr->cmn.ttag.beg.epoch = epoch;
    ihdr->cmn.beg = ihdr->cmn.ttag.beg.epoch + (REAL64)SAN_EPOCH_TO_1970_EPOCH;
    ihdr->cmn.ttag.beg.offset = offset;
    ihdr->tofs = ihdr->cmn.beg;
    ihdr->cmn.ttag.beg.ofis.pps = (ihdr->cmn.ttag.beg.ofis.ext * NANOSEC_PER_SEC - offset)/NANOSEC_PER_OFISTIC;
 
    /* fix end */
    epoch =  ((REAL64) (sys + offset) / (REAL64)NANOSEC_PER_SEC) + duration + EpochYearFactor;
    ihdr->cmn.ttag.end.epoch = epoch;
    ihdr->cmn.end = ihdr->cmn.ttag.end.epoch + (REAL64)SAN_EPOCH_TO_1970_EPOCH;
    ihdr->cmn.ttag.end.offset = offset;
    ihdr->tols = ihdr->cmn.end;
    ihdr->cmn.ttag.end.ofis.pps = (ihdr->cmn.ttag.beg.ofis.ext * NANOSEC_PER_SEC - offset)/NANOSEC_PER_OFISTIC;

    return;
}


void tee2ms_fix_ext(IDA10_TSHDR *ihdr, INT64 delta)
    /* no GPS time string is off by 1s, fix it in command and OFIS tags; delta in s */
{ 
    INT64 tmp;

    tmp = ihdr->cmn.ttag.beg.ext + delta*NANOSEC_PER_SEC;
    ihdr->cmn.ttag.beg.ext = (UINT64)tmp;

    tmp = ihdr->cmn.ttag.end.ext + delta*NANOSEC_PER_SEC;
    ihdr->cmn.ttag.end.ext = (UINT64)tmp;

    tmp = ihdr->cmn.ttag.beg.ofis.ext + delta;
    ihdr->cmn.ttag.beg.ofis.ext = (UINT64)tmp;

    tmp = ihdr->cmn.ttag.end.ofis.ext + delta;
    ihdr->cmn.ttag.end.ofis.ext = (UINT64)tmp;

    return;
}


static BOOL tee2ms_pkt_timing(int i, IDA10_TS *ts, BOOL *new_timeseries, REAL64 *error, BOOL *clock_ok, BOOL *reset_seq_no)
     /* check IDA10 packet for various timing anamomalies, apply known fixes */
     /* return TRUE if packet is to be retained, FALSE to discard */
     /* NOTE: this function is likely to get very messy as timing anomalies are discovered */
{
  int sign;
  UINT64 err;
  BOOL fixed = FALSE, is_seq = TRUE;
  IDA10_TSHDR *ihdr = &ts->hdr;
  BOOL err_ok = ida10TtagIncrErr(&h[i].curr_ihdr, ihdr, &sign, &err);
  BOOL is_leap_sec = FALSE;

  /* print ttag info - original, before any fixes */
  if( debug ) {
    (void)fprintf(stderr, "%s ts_npkt=%ld", h[i].sncl, h[i].ts_npkt+1);
    tee2ms_print_ttag_info(ihdr);
  }

  /* initialize return values */
  *new_timeseries = FALSE;
  *reset_seq_no = FALSE;
  *error = 0.0;

  /* current and previous OFIS timetag params - all values in nanosecs */
  UINT64 curr_ext, curr_sys, curr_pps, curr_offset;
  UINT64 prev_ext, prev_sys, prev_pps, prev_offset;
  IDA10_TSHDR *curr_ihdr, *prev_ihdr;
  IDA10_OFIS_CLOCK_STATUS curr_status, prev_status;
  BOOL curr_status_ok, prev_status_ok;
  curr_ihdr = ihdr;
  if( h[i].ts_npkt > 0 ) {
    prev_ihdr = &h[i].curr_ihdr;
  } else {
    prev_ihdr = ihdr;
  }

  /* clock status for valid offset */
  curr_status = curr_ihdr->cmn.ttag.beg.ofis.status;
  prev_status = prev_ihdr->cmn.ttag.beg.ofis.status;
  curr_status_ok = curr_status.init && curr_status.avail && curr_status.pps;
  prev_status_ok = prev_status.init && prev_status.avail && prev_status.pps;

  /* differences between current and previous timing parameter values */
  tee2ms_get_ttag_info(curr_ihdr, &curr_ext, &curr_sys, &curr_pps, &curr_offset);
  tee2ms_get_ttag_info(prev_ihdr, &prev_ext, &prev_sys, &prev_pps, &prev_offset);
  INT64 diff_ext = curr_ext - prev_ext;
  INT64 diff_sys = curr_sys - prev_sys;
  INT64 diff_pps = curr_pps - prev_pps;
  INT64 diff_offset = curr_offset - prev_offset;

  *clock_ok = curr_status_ok;

  /* repeated packet, discard */
  if( h[i].ts_npkt > 0 && diff_sys == 0 && diff_pps == 0 && diff_ext == 0 && diff_offset == 0 ) {
    if(debug) {
      (void)fprintf(stderr, "Debug: tee2ms_pkt_timing: identical packet\n");
    }
    return FALSE;
  }

  /* discard packets starting at 'sys' value from command line */
  if ( discard_from > 0 && curr_sys >= (discard_from * NANOSEC_PER_OFISTIC) ) {
    if(debug) {
      (void)fprintf(stderr, "Debug: tee2ms_pkt_timing: discarding packet\n");
    }
    return FALSE;
  }

  /* check for sequential 'sys' count, i.e., within one sample interval of expected */
  if(llabs(diff_sys) >= (INT64)(prev_ihdr->nsint * (prev_ihdr->nsamp+1))) {
    if(debug) {
      (void)fprintf(stderr, "Debug: tee2ms_pkt_timing: Unexpected 'sys' time curr_sys=%lld(tics) prev_sys=%lld(tics)\n",
		  curr_sys/NANOSEC_PER_OFISTIC, prev_sys/NANOSEC_PER_OFISTIC);
    }

    /* break in packet sequence */
    is_seq = FALSE;
    *new_timeseries = TRUE;
    *reset_seq_no = TRUE;
    *error = 0.0;
  }

  /* force new timeseries at <sys> specified on command line */
  if ( start_new > 0 && curr_sys == (start_new * NANOSEC_PER_OFISTIC) ) {

    *new_timeseries = TRUE;
    if( !is_seq ) {
      *reset_seq_no = TRUE;
    }
    *error = 0.0;

    /* save current offset if valid */
    h[i].curr_offset = 0;
    if( curr_status_ok ) {
      h[i].curr_offset = curr_offset;
      if(debug) {
	(void)fprintf(stderr, "Debug: tee2ms_pkt_timing: * curr_offset=%lld(tics)\n", h[i].curr_offset/NANOSEC_PER_OFISTIC);
      }
    }

    return TRUE;
  }

  /* Femtometer reboot start */
  if( !is_seq ) {
    h[i].is_sequential = FALSE;

    if( !curr_status.init && !curr_status.avail ) {
      if( !h[i].is_reboot ) {
	h[i].is_reboot = TRUE;
	if(debug) {
	  (void)fprintf(stderr, "Debug: tee2ms_pkt_timing: start of Femtometer reboot ...\n");
	}
      }
      return FALSE; /* discard packet */
    } 

    /* discard packet if 'sys' value lower than previous */
    if ( !h[i].is_reboot && curr_sys < prev_sys ) {
      if(debug) {
	(void)fprintf(stderr, "Debug: tee2ms_pkt_timing: discarding out of sequence packet\n");
      }
      return FALSE;
    }
  }

  if( curr_status_ok ) {

    /* Femtometer reboot end */
    if ( h[i].is_reboot ) {
      h[i].is_reboot = FALSE;
      if(debug) {
	(void)fprintf(stderr, "Debug: end of Femtometer reboot ...\n");
      }
      h[i].first_offset = 0;
      h[i].curr_offset = 0;

      /* first packet after reboot => new timeseries, reset seq# */
      is_seq = FALSE; 
      *new_timeseries = TRUE;
      *reset_seq_no = TRUE;
      *error = 0.0;
    }

    /* save current offset if valid */
    if( h[i].curr_offset == 0 ) {
      h[i].curr_offset = curr_offset;
      if(debug) {
	(void)fprintf(stderr, "Debug: tee2ms_pkt_timing: * curr_offset=%lld(tics)\n", h[i].curr_offset/NANOSEC_PER_OFISTIC);
      }
    }

    /* test if packet start time is within leap sec - may need to use errtol */
    if (is_leap_second_n((double)curr_ihdr->cmn.beg) != NULL ) {
      if(debug) {
	(void)fprintf(stderr, "Debug: tee2ms_pkt_timing: Detected leap sec\n");
      }     
      is_leap_sec = TRUE;
    } 

    /* packet sequential, clock flags ok */
    if( is_seq && !is_leap_sec ) {

      /* 'ext' (GPS time) unchanged but 'pps' advances ~1s or 'ext' decreases by 1s but 'pps' unchanged */
      if( !h[i].large_offset && (diff_ext == 0 && llabs(diff_pps - (INT64)(NANOSEC_PER_SEC)) < 2*NANOSEC_PER_OFISTIC )
	  || ( llabs(diff_pps) < 2*NANOSEC_PER_OFISTIC && diff_ext == (INT64)(-NANOSEC_PER_SEC) )) { 
	if(debug) {
	  (void)fprintf(stderr, "Debug: tee2ms_pkt_timing: ext is 1(s) behind, fixing timing for %s ...\n", h[i].sncl);
	}
	/* advance GPS clock by 1s */
	tee2ms_fix_ext(curr_ihdr, 1);

	/* increase offset temporarily by 1s */
	tee2ms_fix_timestamp(curr_ihdr, curr_sys, curr_offset + NANOSEC_PER_SEC);
	fixed = TRUE;

	curr_offset += NANOSEC_PER_SEC;
	diff_offset = 0;
      }

      /* large offset change => ext or PPS value, use previous offset and fix timing */ 
      if( llabs(diff_offset) >= errtol*NANOSEC_PER_SEC && prev_status.init ) {
	/* corner case where first pkt in file has bad time due to 'ext' being 1s behind, e.g., 2009.353 */
	if( h[i].chan_npkt == 1 && diff_ext == (INT64)(NANOSEC_PER_SEC) && diff_pps < 2*NANOSEC_PER_OFISTIC ) { 
	  h[i].first_offset = 0;
	  if(debug) {
	    (void)fprintf(stderr, "Debug: tee2ms_pkt_timing: First pkt 'ext' for %s 1s behind, setting first pkt offset to 0 ...\n",
		    h[i].sncl);
	  }	
	} else {
	  if(debug) {
	    (void)fprintf(stderr, "Debug: tee2ms_pkt_timing: Large offset change, diff_offset=%lld(tics), fixing timing for %s using prev offset %lld(tics) ...\n",
		    diff_offset/NANOSEC_PER_OFISTIC, h[i].sncl, prev_offset/NANOSEC_PER_OFISTIC);
	  }
	  tee2ms_fix_timestamp(curr_ihdr, curr_sys, prev_offset);
	  fixed = TRUE;
	  h[i].is_questionable_ttag = TRUE;
	  h[i].large_offset = TRUE;
	  curr_offset = prev_offset;
	  diff_offset = 0;
	}
      } else {
	h[i].large_offset = FALSE;
      }
    }

    /* store latest valid offset for this timeseries */
    if ( h[i].curr_offset != curr_offset ) {
      h[i].curr_offset = curr_offset;
      if(debug) {
	(void)fprintf(stderr, "Debug: tee2ms_pkt_timing: ** curr_offset=%lld(tics)\n", h[i].curr_offset/NANOSEC_PER_OFISTIC);
      }
    }

    /* update offset for first packet in timeseries */
    if ( is_seq && h[i].first_offset == 0 && h[i].curr_offset != 0 && h[i].first_offset != h[i].curr_offset ) {
      h[i].first_offset = h[i].curr_offset;
      if (debug) {
	(void)fprintf(stderr, "Debug: tee2ms_pkt_timing: Setting timeseries start time for %s using offset %lld(tics) ...\n",
		  h[i].sncl, h[i].first_offset/NANOSEC_PER_OFISTIC);
      }
      tee2ms_fix_timestamp(&h[i].first_ihdr, h[i].first_ihdr.cmn.ttag.beg.sys, h[i].first_offset);
      if( !prev_status_ok ) {
	h[i].is_questionable_ttag = TRUE;

	/* fix max|error| time - not critical, only used for file stats */
	if(debug) {
	  (void)fprintf(stderr, "Debug: tee2ms_pkt_timing: Fixing max|error| time for %s using offset %lld(tics) ...\n",
		  h[i].sncl, h[i].curr_offset/NANOSEC_PER_OFISTIC);
	}
	tee2ms_fix_timestamp(&h[i].error.pkt, h[i].error.pkt.cmn.ttag.beg.sys, h[i].curr_offset);
        h[i].error.value = fabsl(h[i].error.pkt.cmn.beg - (h[i].first_ihdr.cmn.beg + h[i].error.ts_nsamp * h[i].error.pkt.sint));
	h[i].error.clock_ok = TRUE;
      }
    }
    
  }

  /* packet sequential, clock flags don't need to be ok */
  if ( is_seq ) {

    /* no GPS/PPS time available, timestamps usually, but not always, unknown (1999.xxx) */
    if ( (!curr_status.avail || !curr_status.pps) ) {
      /* use last valid offset */
      if( h[i].curr_offset != 0 && diff_offset != 0 ) {
	if(debug) {
	  (void)fprintf(stderr, "Debug: GPS/PPS unavailable, fixing timing for %s using offset %lld(tics) ...\n",
		    h[i].sncl, h[i].curr_offset/NANOSEC_PER_OFISTIC);
	}
	tee2ms_fix_timestamp(curr_ihdr, curr_sys, h[i].curr_offset);
	fixed = TRUE;
	curr_offset = h[i].curr_offset;
	diff_offset = 0;
	h[i].is_questionable_ttag = TRUE;
      }
    }
      
    /* apply externally supplied init offset */
    if( h[i].first_offset != 0 && h[i].first_offset != h[i].curr_offset && ( h[i].curr_offset == 0 || h[i].chan_npkt == 0 ) ) {
	if(debug) {
	  (void)fprintf(stderr, "Debug: fixing timing for %s using external offset %lld(tics) ...\n",
		    h[i].sncl, h[i].first_offset/NANOSEC_PER_OFISTIC);
	}
	tee2ms_fix_timestamp(curr_ihdr, curr_sys, h[i].first_offset);
	/* fix stored first packet timing so error will be correctly computed */
	if(h[i].ts_npkt == 0) {
	  tee2ms_fix_timestamp(&h[i].first_ihdr, curr_sys, h[i].first_offset);
	}
	fixed = TRUE;
	curr_offset = h[i].first_offset;
	diff_offset = curr_offset - prev_offset;
	if ( h[i].curr_offset != curr_offset ) {
	  h[i].curr_offset = curr_offset;
	  if(debug) {
	    (void)fprintf(stderr, "Debug: tee2ms_pkt_timing: ** curr_offset=%lld(tics)\n", h[i].curr_offset/NANOSEC_PER_OFISTIC);
	  }
	}
	h[i].is_questionable_ttag = TRUE;
     }
     
     /* determine cumulative error (actual - expected) time for this packet;
       for expected time use nominal sample rate */
     *error = curr_ihdr->cmn.beg - (h[i].first_ihdr.cmn.beg + h[i].ts_nsamp * curr_ihdr->sint);

     if( timing_opt == 1 && ( curr_status_ok || fixed ) ) {
       /* new_timeseries if cumulative error if adding this packet would exceed tolerence */
       if( fabsl(*error) >= errtol ) {
	 if(debug) {
	   (void)fprintf(stderr, "Debug: tee2ms_pkt_timing: if pkt added, |error|=|%f| >= %f\n", *error, errtol);
	 }
	 *new_timeseries = TRUE;
	 *reset_seq_no = FALSE;
	 *error = 0.0;
       }
     }
   }
  
  /* print current packet timing info if it was corrected */
  if( fixed ) {
    if( debug ) {
      (void)fprintf(stderr, "----- fixed -----\n");
      (void)fprintf(stderr, "%s ts_npkt=%ld", h[i].sncl, h[i].ts_npkt+1);
      tee2ms_print_ttag_info(ihdr);
      (void)fprintf(stderr, "-----------------\n");
    }    
  }

  return TRUE;
}


static void tee2ms_file_print_stats(char *file_name)
     /* print timeseris statistics */
{
  char time_str[MAXSTR];
  int i;

  (void)fprintf(stderr, "File Stats: file=%s\n", file_name);
  for(i=0; i < num_sncl; i++) {
    (void)fprintf(stderr, " %s #samples=%ld", h[i].sncl, h[i].chan_nsamp);
    (void)sprintf(time_str, "%s", time_to_str(nepoch_to_int(h[i].error.pkt.cmn.beg), date_fmt));
    if( !h[i].error.clock_ok ) {
      (void)fprintf(stderr, " max|error|=n/a@%s", time_str);
    } else {
      (void)fprintf(stderr, " max|error|=%f(s)@%s", h[i].error.value, time_str);
    }
    if(h[i].is_sequential) {
      (void)fprintf(stderr, " is_sequential=YES");
    } else {
      (void)fprintf(stderr, " is_sequential=NO");
    }
    (void)fprintf(stderr, " first_valid_offset=%lld", h[i].first_valid_offset/NANOSEC_PER_OFISTIC);
    (void)fprintf(stderr, " last_valid_offset=%lld", h[i].last_valid_offset/NANOSEC_PER_OFISTIC);
    (void)fprintf(stderr, "\n");
  }

  return;
}


static int tee2ms_cvt_and_out(int i)
     /* convert to mini-SEED and output */
{
  int npacked, nblocks = 0, status, ok;
  size_t nwritten;
  long num_written;
  char *p_ms = NULL;
  int *p_data_int;
  float *p_data_float;
  double *p_data_double;

  int blockette_space;	/* # of bytes required for blockettes.	*/
  BS *bs;		/* ptr to blockette structure.		*/
  BS *last_bs;		/* ptr to last permanent blockette.	*/
  MS_ATTR attr;
  int align;

  if( h[i].first_offset == 0 ) {
    if(debug) {
      (void)fprintf(stderr, "Debug: tee2ms_cvt_and_out: %s #packets=%ld first_offset=%lld(tics), cannot fix start time, discarding timeseries\n",
		 h[i].sncl, h[i].ts_npkt, h[i].first_offset);
    }
    return 0;
  }

  /* (clock) quality settings */
  if(h[i].is_questionable_ttag) {
    h[i].shdr->data_quality_flags |= QUALITY_QUESTIONABLE_TIMETAG;
  }

#ifdef DEBUG
  for (bs=h[i].shdr->pblockettes; bs!=NULL; bs=bs->next) {
    if (bs->type == 1001) {
         fprintf(stderr, "Debug: tee2ms_cvt_and_out: Blockette 1001 exists!\n");
    }
    if (bs->type == 100) {
         fprintf(stderr, "Debug: tee2ms_cvt_and_out: Blockette 100 exists!\n");
    }
  }
#endif

  /* add blockettes if not already added */
  /* they would be already added if new timeseries is started due to time tears */
  /* don't bother finding specific blockette types as we add just these 2 types */
  /* blockette 1001 and 100 info are updated by ms_pack2_* functions */
  if( h[i].shdr->num_blockettes == 0 && h[i].shdr->pblockettes == NULL ) {

#ifdef FOO
    ok = tee2ms_add_blockette_1001(h[i].shdr, i);
    if( !ok ) {
      fprintf(stderr, "Error: Unable to create blockette 1001\n");
      return MS_ERROR;
    } else {
      if(debug) {
        (void)fprintf(stderr, "Debug: tee2ms_cvt_and_out: Added blockette 1001 for hash index, i=%d\n", i); 
      }
    }
#endif
 
    if(timing_opt == 0) {
      ok = tee2ms_add_blockette_100(h[i].shdr, i);
      if( !ok ) {
	fprintf(stderr, "Error: Unable to create blockette 100\n");
	return MS_ERROR;
      }
    }
  }

  /* determine space required for all blockettes.		*/
  for (bs=h[i].shdr->pblockettes, blockette_space=0, last_bs=NULL; 
    bs!=NULL; last_bs=bs, bs=bs->next) {
     blockette_space += bs->len + ((bs->len%4) ? 4-(bs->len%4) : 0);
  }

  /* # data frames is needed later by init_sdr_hdr() to update blockette 1001 frame count field */
  attr = get_ms_attr(h[i].shdr);
  if (attr.alignment == 0) return (MS_ERROR);
  align = attr.alignment;
  h[i].shdr->first_data = ((sizeof(SDR_HDR)+blockette_space+align-1)/align)*align;

  switch(h[i].shdr->data_type) {
  case STEIM2:
    h[i].shdr->num_data_frames = (h[i].shdr->blksize - h[i].shdr->first_data)/sizeof(FRAME);
    status = ms_pack2_data (h[i].shdr, NULL, h[i].nout, h[i].databuf.p_int, &nblocks, &npacked, &p_ms, 0, NULL);
    break;

  case IEEE_FP_SP:
    status = ms_pack2_data (h[i].shdr, NULL, h[i].nout, h[i].databuf.p_float, &nblocks, &npacked, &p_ms, 0, NULL);
    break;

  case IEEE_FP_DP:
    status = ms_pack2_data (h[i].shdr, NULL, h[i].nout, h[i].databuf.p_double, &nblocks, &npacked, &p_ms, 0, NULL);
    break;

  default:
    (void)fprintf (stderr, "Error: tee2ms_cvt_and_out: unknown data type %d\n", h[i].shdr->data_type);
    status = 1;
    break;
  }

  if(status == 0) {
     nwritten = fwrite (p_ms, nblocks*mseed_blksize, 1, h[i].outf);

     if (nwritten != (size_t)1) {
	(void)fprintf (stderr, "Error: tee2ms_cvt_and_out: Writing miniseed output, errno=%d\n", errno);
	status = 1;
     } else {
       if(debug) {
	 (void)fprintf(stderr, "Debug: tee2ms_cvt_and_out: %s: output: #mSEED blocks = %d, #samples packed = %d, av_rate=%f",
		       h[i].sncl, nblocks, npacked, h[i].av_rate);
	 if( h[i].is_questionable_ttag ) {
	   (void)fprintf(stderr, ", questionable_ttag=TRUE\n");
	 } else { 
	   (void)fprintf(stderr, ", questionable_ttag=FALSE\n");
	 }
       }
     }
 
     if(p_ms) {
	free(p_ms);
	p_ms = NULL;
     }

     /* default seq_no to use for next call to ms_pack2()_data */
     h[i].shdr->seq_no += nblocks;
  }

  return status;
}


static void tee2ms_alloc_data(int i)
     /* check and allocate memory for accumulated data */
{
    size_t nalloc = h[i].nalloc;

    /* check if there's space left */
    if(((int)nalloc > (h[i].nout + (int)h[i].curr_ihdr.nsamp))) {
      return;
    }

    /* allocate memory for data, store all integer data as 32-bit for mSEED packing */
    switch(h[i].curr_ihdr.datatype) {
      case IDA10_DATA_INT32:
      case IDA10_DATA_INT16:
      case IDA10_DATA_INT8: 
	if(h[i].databuf.p_int == NULL && nalloc == 0) {
	  nalloc = DEF_OUTDATA;
	  h[i].databuf.p_int = (int*)calloc(nalloc, sizeof(int));
	} else {
	  nalloc = h[i].nalloc + CHUNK_SIZE;
	  h[i].databuf.p_int = (int*)realloc(h[i].databuf.p_int, sizeof(int)*nalloc);
	}
	if(h[i].databuf.p_int == NULL) {
	  (void)fprintf (stderr, "Error: Unable to allocate memory for (integer) data buffer\n");
	  exit(1);
	}
	break;
      case IDA10_DATA_REAL64:
	if(h[i].databuf.p_double == NULL && nalloc == 0) {
	  nalloc = DEF_OUTDATA;
	  h[i].databuf.p_double = (double*)calloc(nalloc, sizeof(double));
	} else {
	  nalloc = h[i].nalloc + CHUNK_SIZE;
	  h[i].databuf.p_double = (double*)realloc(h[i].databuf.p_double, sizeof(double)*nalloc);
	}
	if(h[i].databuf.p_double == NULL) {
	  (void)fprintf (stderr, "Error: Unable to allocate memory for (double) data buffer\n");
	  exit(1);
	}
	break;
      case IDA10_DATA_REAL32:
	if(h[i].databuf.p_float == NULL && nalloc == 0) {
	  nalloc = DEF_OUTDATA;	  
	  h[i].databuf.p_float = (float*)calloc(nalloc, sizeof(float));
	} else {
	  nalloc = h[i].nalloc + CHUNK_SIZE;
	  h[i].databuf.p_float = (float*)realloc(h[i].databuf.p_float, sizeof(float)*nalloc);
	}
	if(h[i].databuf.p_float == NULL) {
	  (void)fprintf (stderr, "Error: Unable to allocate memory for (float) data buffer\n");
	  exit(1);
	}
	break;
      case IDA10_DATA_UNKNOWN:
      default:
	(void)fprintf (stderr, "Error: Unknown data type %d\n", h[i].curr_ihdr.datatype);
	exit(1);	
    }

    h[i].nalloc = nalloc;

    return;
}


static void tee2ms_pkt_add_hdr(int i, IDA10_TSHDR *ihdr)
     /* add/update header info for each new packet */
{
  INT_TIME starttime, endtime;
  int status = 0;
  double duration;
  REAL64 error;

  if(h[i].ts_npkt > 0) {
    h[i].prev_ihdr = h[i].curr_ihdr;
  } else {
    h[i].first_ihdr = *ihdr;
    h[i].prev_ihdr = *ihdr;
  }
  h[i].curr_ihdr = *ihdr;
  starttime = nepoch_to_int(h[i].first_ihdr.cmn.beg);

  /* update MSEED hdr for possibly fixed sart times */
  h[i].shdr->hdrtime = nepoch_to_int(h[i].first_ihdr.cmn.beg);
  h[i].shdr->begtime = h[i].shdr->hdrtime;  

  /* IDA10 packet 'end' is time at START of last sample in packet, so ... */
  endtime = nepoch_to_int(h[i].curr_ihdr.cmn.end + h[i].curr_ihdr.sint); /* approx */
  h[i].ts_nsamp += h[i].curr_ihdr.nsamp;
  h[i].chan_nsamp += h[i].curr_ihdr.nsamp;
  duration = (SEED_FLOAT)tdiff(endtime, starttime)/USECS_PER_SEC;
  h[i].ts_npkt++;
  h[i].chan_npkt++;  
  h[i].av_rate = (SEED_FLOAT)(h[i].ts_nsamp)/(SEED_FLOAT)duration;

  return;
}


static void tee2ms_pkt_add_data(int i, IDA10_TSDAT data)
     /* add data to data buffer for each new packet */
{
  int j, nout;

  tee2ms_alloc_data(i);
  
  nout = h[i].nout;
  switch(h[i].curr_ihdr.datatype) {
  case IDA10_DATA_INT32:
    for (j=0; j < h[i].curr_ihdr.nsamp; j++) {
      h[i].databuf.p_int[nout++] = (int)data.int32[j];
    }
    break;
  case IDA10_DATA_INT16:
    for (j=0; j < h[i].curr_ihdr.nsamp; j++) {
      h[i].databuf.p_int[nout++] = (int)data.int16[j];
    }
    break;
  case IDA10_DATA_INT8:
    for (j=0; j < h[i].curr_ihdr.nsamp; j++) {
      h[i].databuf.p_int[nout++] = (int)data.int8[j];
    }
    break;
  case IDA10_DATA_REAL64:
    for (j=0; j < h[i].curr_ihdr.nsamp; j++) {
      h[i].databuf.p_double[nout++] = (double)data.real64[j];
    }
    break;
  case IDA10_DATA_REAL32:
    for (j=0; j < h[i].curr_ihdr.nsamp; j++) {
      h[i].databuf.p_float[nout++] = (double)data.real32[j];
    }
    break;
  case IDA10_DATA_UNKNOWN:
    break;
  default:
    break;
  }

  h[i].nout = nout;

  return;
}


static void tee2ms_pkt_add(int i, IDA10_TS ts, REAL64 error, BOOL clock_ok)
     /* add data to data buffer */
{
  tee2ms_pkt_add_hdr(i, &ts.hdr);
  tee2ms_pkt_add_data(i, ts.data);

  /* update timing error for this channel over entire input file's packets */
  if( fabsl(error) > h[i].error.value ) {
    h[i].error.pkt = ts.hdr;
    h[i].error.value = fabsl(error);
    h[i].error.ts_nsamp = h[i].ts_nsamp - h[i].curr_ihdr.nsamp;
    h[i].error.clock_ok = clock_ok;
  }

  /* store first valid offset for file */
  if( h[i].first_valid_offset == 0 ) {
    h[i].first_valid_offset = h[i].curr_offset;
  }
 
  if(debug) {
    (void)fprintf(stderr, "Debug: tee2ms_pkt_add: i=%d ts_npkt=%d type=%d cum_error=%f(s) max|error|=%f(s)",
		  i, h[i].ts_npkt, ts.hdr.cmn.type, error, h[i].error.value);
    if( h[i].is_questionable_ttag ) {
      (void)fprintf(stderr, ", questionable_ttag=TRUE");
    } else { 
      (void)fprintf(stderr, ", questionable_ttag=FALSE");
    }
    if( h[i].large_offset ) {
      (void)fprintf(stderr, ", large_offset=TRUE");
    } else { 
      (void)fprintf(stderr, ", large_offset=FALSE");
    }
    (void)fprintf(stderr, "\n");
  }     

  return;
}


static void tee2ms_write(char *file_name) {
  int i;

  if(debug) {
    (void)fprintf(stderr, "Debug: tee2ms_write: Writing remaining buffered data for file %s ...\n", file_name);
  }

  /* write pending data */
  for(i=0; i < num_sncl; i++) {

    /* if necessary open output file */
    if(h[i].nout > 0) {
      tee2ms_open_output_file(i);
    }

    /* store last valid offset for file */
    if( h[i].last_valid_offset == 0 ) {
      h[i].last_valid_offset = h[i].curr_offset;
    }  

    if(h[i].nout > 0) {
      (void)tee2ms_cvt_and_out(i);
    }

    /* close output file */
    tee2ms_close_output_file(i);
  }

  return;
}


static void tee2ms_clean_up() {
    int i;

    if(debug) {
      (void)fprintf(stderr, "Debug: tee2ms_clean_up: Cleaning up ...\n");
    }

    for(i = 0; i < num_sncl; i++) {

      /* some mapped channels may have no data so check for non NULL pointer first */
      if( h[i].shdr == NULL ) {
	continue;
      }

      /* free data buffer */
      switch(h[i].shdr->data_type) {
      case STEIM2:
	if (h[i].databuf.p_int) {
	  free(h[i].databuf.p_int);
	}
	break;
      case IEEE_FP_SP:
	if (h[i].databuf.p_float) {
	  free(h[i].databuf.p_float);
	}
	break;	
      case IEEE_FP_DP:
	if( h[i].databuf.p_double ) {
	  free(h[i].databuf.p_double);
	}
	break;
      default:
	break;
      }
      free_data_hdr(h[i].shdr);

      /* free hash key */
      free(h[i].key);

      tee2ms_hash_reset(i);
    }

    /* free up hash table */
    hdestroy();

    return;
}


static int tee2ms_process_file(FILE *input, char *file_name)
    /* convert input file to mSEED */
{
  IDA10_TS ts;
  int i=-1, status;
  BOOL ok, new_timeseries, reset_seq_no, clock_ok;
  REAL64 error;

  while (tee2ms_pkt_read(input, &ts)) {

    /* initialize channel header times, etc. */
    i = tee2ms_hash_update(&ts.hdr);
    if( i < 0 || i > MAX_SNCL ) {
      continue;
    }

    /* first packet of new input file */
    if( h[i].is_first_pkt ) {
      tee2ms_reset_timeseries( i, &ts.hdr, TRUE, TRUE );
      h[i].is_first_pkt = FALSE;
    }

    /* analyze packet timing parameters */
    ok = tee2ms_pkt_timing(i, &ts, &new_timeseries, &error, &clock_ok, &reset_seq_no);

    if(!ok) { /* discard packet */
      if(debug) {
	(void)fprintf(stderr, "Debug: tee2ms_process_file: discarding packet\n");
        (void)fprintf(stderr, "\n"); /* packet separator in debug output msgs */
      }
      continue;
    }

    if(new_timeseries) {
      /* if necessary open output file */
      tee2ms_open_output_file(i);
      /* new time series => data flush, timeseries reset */
      status = tee2ms_cvt_and_out(i);
      if(status == MS_ERROR) {
	return status;
      }
      tee2ms_reset_timeseries( i, &ts.hdr, clock_ok, reset_seq_no );
    }

    tee2ms_pkt_add( i, ts, error, clock_ok );

    if(debug) {
      (void)fprintf(stderr, "\n"); /* packet separator in debug output msgs */
    }
  }

  /* write remaining output and close file(s) */
  tee2ms_write(file_name);

  /* print file statistics */
  if( debug || statistics ) {
    tee2ms_file_print_stats(file_name);
  }

  return 0;
}


int main(int argc, char **argv)
{
    FILE *fp;
    char buffer[2 * IDA10_MAXRECLEN];
    char cmd[256];
    int status, return_status = 0;

    tee2ms_cmdlineargs(argc, argv);
    tee2ms_init();

    /* skip over all options and their arguments */
    argv = &(argv[optind]);
    argc -= optind;

    if (argc == 0) {
      /* read from stdin */
      return_status = tee2ms_process_file(stdin, "");
    } else {
      /* one or more input files */
      char file_name[MAXSTR];
      int file_num = 0;
      BOOL done = FALSE;

      while(!done) {
	strcpy(file_name, argv[file_num]);
	if(warn) {
	  (void)fprintf(stderr, "Processing file %s ...\n", file_name);
	}

	if(no_gzip) {
	  /* input files not gzip'd */
	  if(!(fp = fopen(file_name, "r"))) {
	    if(warn) {
	      (void)fprintf (stderr, "Warning: Cannot open file %s, ignoring\n", file_name);
	    }
	  } else {
	    status = tee2ms_process_file(fp, file_name);
	    (void)fclose(fp);
	  }	  
	} else {
	  /* file assumed to be gzip'd */
	  if (strcmp(file_name + strlen(file_name) - 3, ".gz") != 0) {
	    /* skip *.gz.incomplete and such */
	    if(strstr(file_name, ".gz")) {
	      if(warn) {
		(void)fprintf (stderr, "Warning: Skipping file %s\n", file_name);
	      }
	      continue;
	    }
	    /* No .gz: tack one on */
	    (void)strcat(file_name, ".gz");
	  }

	  char tmpfile[256];
	  sprintf(tmpfile, "/tmp/tee2ms%d", getpid());

	  (void)sprintf(cmd, "/bin/gunzip -c %s > %s", file_name, tmpfile);
	  if(system(cmd) < 0) {
	    perror("Error");
	    exit(1);
	  }

	  if(!(fp = fopen(tmpfile, "r"))) {
	    if(warn) {
	      (void)fprintf (stderr, "Warning: cannot open file %s, ignoring\n", tmpfile);
	    }
	  } else {
	    status = tee2ms_process_file(fp, tmpfile);
	    (void)fclose(fp);
	    (void)unlink(tmpfile);
	  }
 	}	

	if(status) {
	  return_status = 1;
	}

	file_num++;
	if ( file_num == argc ) {
	  done = TRUE;
	}
      }
    }

    tee2ms_clean_up();

    return return_status;
}
